@@ -132,9 +132,9 @@ static BOOL rdp_check_monitor_layout_pdu_state_(const rdpRdp* rdp, BOOL expected
 		const DWORD log_level = WLOG_ERROR;
 		if (WLog_IsLevelActive(rdp->log, log_level))
 		{
-			WLog_PrintMessage(rdp->log, WLOG_MESSAGE_TEXT, log_level, line, file, fkt,
-			                  "Expected rdp->monitor_layout_pdu == %s",
-			                  expected ? "TRUE" : "FALSE");
+			WLog_PrintTextMessage(rdp->log, log_level, line, file, fkt,
+			                      "Expected rdp->monitor_layout_pdu == %s",
+			                      expected ? "TRUE" : "FALSE");
 		}
 		return FALSE;
 	}
@@ -153,8 +153,8 @@ static BOOL rdp_set_monitor_layout_pdu_state_(rdpRdp* rdp, BOOL value, const cha
 		const DWORD log_level = WLOG_WARN;
 		if (WLog_IsLevelActive(rdp->log, log_level))
 		{
-			WLog_PrintMessage(rdp->log, WLOG_MESSAGE_TEXT, log_level, line, file, fkt,
-			                  "rdp->monitor_layout_pdu == TRUE, expected FALSE");
+			WLog_PrintTextMessage(rdp->log, log_level, line, file, fkt,
+			                      "rdp->monitor_layout_pdu == TRUE, expected FALSE");
 		}
 		return FALSE;
 	}
@@ -202,8 +202,15 @@ BOOL rdp_read_security_header(rdpRdp* rdp, wStream* s, UINT16* flags, UINT16* le
 	if (!Stream_CheckAndLogRequiredLengthWLog(rdp->log, s, 4))
 		return FALSE;
 
-	Stream_Read_UINT16(s, *flags); /* flags */
-	Stream_Seek(s, 2);             /* flagsHi (unused) */
+	*flags = Stream_Get_UINT16(s);                 /* flags */
+	const uint16_t flagsHi = Stream_Get_UINT16(s); /* flagsHi (unused) */
+	if ((*flags & SEC_FLAGSHI_VALID) != 0)
+	{
+		WLog_Print(rdp->log, WLOG_WARN,
+		           "[MS-RDPBCGR] 2.2.8.1.1.2.1 Basic (TS_SECURITY_HEADER) SEC_FLAGSHI_VALID field "
+		           "set: flagsHi=0x%04" PRIx16,
+		           flagsHi);
+	}
 	WLog_Print(rdp->log, WLOG_TRACE, "%s",
 	           rdp_security_flag_string(*flags, buffer, sizeof(buffer)));
 	if (length)
@@ -232,8 +239,9 @@ BOOL rdp_write_security_header(rdpRdp* rdp, wStream* s, UINT16 flags)
 
 	WLog_Print(rdp->log, WLOG_TRACE, "%s", rdp_security_flag_string(flags, buffer, sizeof(buffer)));
 	/* Basic Security Header */
-	Stream_Write_UINT16(s, flags); /* flags */
-	Stream_Write_UINT16(s, 0);     /* flagsHi (unused) */
+	WINPR_ASSERT((flags & SEC_FLAGSHI_VALID) == 0); /* SEC_FLAGSHI_VALID is unsupported */
+	Stream_Write_UINT16(s, flags);                  /* flags */
+	Stream_Write_UINT16(s, 0);                      /* flagsHi (unused) */
 	return TRUE;
 }
 
@@ -268,9 +276,9 @@ BOOL rdp_read_share_control_header(rdpRdp* rdp, wStream* s, UINT16* tpktLength,
 
 		char buffer[128] = { 0 };
 		WLog_Print(rdp->log, WLOG_DEBUG,
-		           "[Flow control PDU] type=%s, tpktLength=%" PRIuz ", remainingLength=%" PRIuz,
-		           pdu_type_to_str(*type, buffer, sizeof(buffer)), tpktLength ? *tpktLength : 0,
-		           remainingLength ? *remainingLength : 0);
+		           "[Flow control PDU] type=%s, tpktLength=%" PRIu16 ", remainingLength=%" PRIu16,
+		           pdu_type_to_str(*type, buffer, sizeof(buffer)), tpktLength ? *tpktLength : 0u,
+		           remainingLength ? *remainingLength : 0u);
 		return TRUE;
 	}
 
@@ -303,7 +311,7 @@ BOOL rdp_read_share_control_header(rdpRdp* rdp, wStream* s, UINT16* tpktLength,
 		*channel_id = 0; /* Windows XP can send such short DEACTIVATE_ALL PDUs. */
 
 	char buffer[128] = { 0 };
-	WLog_Print(rdp->log, WLOG_DEBUG, "type=%s, tpktLength=%" PRIuz ", remainingLength=%" PRIuz,
+	WLog_Print(rdp->log, WLOG_DEBUG, "type=%s, tpktLength=%" PRIu16 ", remainingLength=%" PRIuz,
 	           pdu_type_to_str(*type, buffer, sizeof(buffer)), len, remLen);
 	if (remainingLength)
 	{
@@ -375,16 +383,17 @@ BOOL rdp_write_share_data_header(rdpRdp* rdp, wStream* s, size_t length, BYTE ty
 	Stream_Write_UINT8(s, STREAM_LOW); /* streamId (1 byte) */
 	Stream_Write_UINT16(
 	    s, WINPR_ASSERTING_INT_CAST(uint16_t, length)); /* uncompressedLength (2 bytes) */
-	Stream_Write_UINT8(s, type);       /* pduType2, Data PDU Type (1 byte) */
-	Stream_Write_UINT8(s, 0);          /* compressedType (1 byte) */
-	Stream_Write_UINT16(s, 0);         /* compressedLength (2 bytes) */
+	Stream_Write_UINT8(s, type);                        /* pduType2, Data PDU Type (1 byte) */
+	Stream_Write_UINT8(s, 0);                           /* compressedType (1 byte) */
+	Stream_Write_UINT16(s, 0);                          /* compressedLength (2 bytes) */
 	return TRUE;
 }
 
-static BOOL rdp_security_stream_init(rdpRdp* rdp, wStream* s, BOOL sec_header)
+static BOOL rdp_security_stream_init(rdpRdp* rdp, wStream* s, BOOL sec_header, UINT16* sec_flags)
 {
 	WINPR_ASSERT(rdp);
 	WINPR_ASSERT(s);
+	WINPR_ASSERT(sec_flags);
 
 	if (rdp->do_crypt)
 	{
@@ -397,12 +406,12 @@ static BOOL rdp_security_stream_init(rdpRdp* rdp, wStream* s, BOOL sec_header)
 				return FALSE;
 		}
 
-		rdp->sec_flags |= SEC_ENCRYPT;
+		*sec_flags |= SEC_ENCRYPT;
 
 		if (rdp->do_secure_checksum)
-			rdp->sec_flags |= SEC_SECURE_CHECKSUM;
+			*sec_flags |= SEC_SECURE_CHECKSUM;
 	}
-	else if (rdp->sec_flags != 0 || sec_header)
+	else if (*sec_flags != 0 || sec_header)
 	{
 		if (!Stream_SafeSeek(s, 4))
 			return FALSE;
@@ -411,7 +420,7 @@ static BOOL rdp_security_stream_init(rdpRdp* rdp, wStream* s, BOOL sec_header)
 	return TRUE;
 }
 
-wStream* rdp_send_stream_init(rdpRdp* rdp)
+wStream* rdp_send_stream_init(rdpRdp* rdp, UINT16* sec_flags)
 {
 	wStream* s = NULL;
 
@@ -426,7 +435,7 @@ wStream* rdp_send_stream_init(rdpRdp* rdp)
 	if (!Stream_SafeSeek(s, RDP_PACKET_HEADER_MAX_LENGTH))
 		goto fail;
 
-	if (!rdp_security_stream_init(rdp, s, FALSE))
+	if (!rdp_security_stream_init(rdp, s, FALSE, sec_flags))
 		goto fail;
 
 	return s;
@@ -435,9 +444,9 @@ fail:
 	return NULL;
 }
 
-wStream* rdp_send_stream_pdu_init(rdpRdp* rdp)
+wStream* rdp_send_stream_pdu_init(rdpRdp* rdp, UINT16* sec_flags)
 {
-	wStream* s = rdp_send_stream_init(rdp);
+	wStream* s = rdp_send_stream_init(rdp, sec_flags);
 
 	if (!s)
 		return NULL;
@@ -451,9 +460,9 @@ fail:
 	return NULL;
 }
 
-wStream* rdp_data_pdu_init(rdpRdp* rdp)
+wStream* rdp_data_pdu_init(rdpRdp* rdp, UINT16* sec_flags)
 {
-	wStream* s = rdp_send_stream_pdu_init(rdp);
+	wStream* s = rdp_send_stream_pdu_init(rdp, sec_flags);
 
 	if (!s)
 		return NULL;
@@ -493,7 +502,7 @@ BOOL rdp_set_error_info(rdpRdp* rdp, UINT32 errorInfo)
 			}
 		}
 		else
-			WLog_Print(rdp->log, WLOG_ERROR, "missing context=%p", context);
+			WLog_Print(rdp->log, WLOG_ERROR, "missing context=%p", (void*)context);
 	}
 	else
 	{
@@ -503,7 +512,7 @@ BOOL rdp_set_error_info(rdpRdp* rdp, UINT32 errorInfo)
 	return TRUE;
 }
 
-wStream* rdp_message_channel_pdu_init(rdpRdp* rdp)
+wStream* rdp_message_channel_pdu_init(rdpRdp* rdp, UINT16* sec_flags)
 {
 	wStream* s = NULL;
 
@@ -517,7 +526,7 @@ wStream* rdp_message_channel_pdu_init(rdpRdp* rdp)
 	if (!Stream_SafeSeek(s, RDP_PACKET_HEADER_MAX_LENGTH))
 		goto fail;
 
-	if (!rdp_security_stream_init(rdp, s, TRUE))
+	if (!rdp_security_stream_init(rdp, s, TRUE, sec_flags))
 		goto fail;
 
 	return s;
@@ -665,7 +674,7 @@ BOOL rdp_read_header(rdpRdp* rdp, wStream* s, UINT16* length, UINT16* channelId)
  * @return \b TRUE for success, \b FALSE otherwise
  */
 
-BOOL rdp_write_header(rdpRdp* rdp, wStream* s, size_t length, UINT16 channelId)
+BOOL rdp_write_header(rdpRdp* rdp, wStream* s, size_t length, UINT16 channelId, UINT16 sec_flags)
 {
 	WINPR_ASSERT(rdp);
 	WINPR_ASSERT(rdp->settings);
@@ -677,8 +686,7 @@ BOOL rdp_write_header(rdpRdp* rdp, wStream* s, size_t length, UINT16 channelId)
 	DomainMCSPDU MCSPDU = (rdp->settings->ServerMode) ? DomainMCSPDU_SendDataIndication
 	                                                  : DomainMCSPDU_SendDataRequest;
 
-	if ((rdp->sec_flags & SEC_ENCRYPT) &&
-	    (rdp->settings->EncryptionMethods == ENCRYPTION_METHOD_FIPS))
+	if ((sec_flags & SEC_ENCRYPT) && (rdp->settings->EncryptionMethods == ENCRYPTION_METHOD_FIPS))
 	{
 		const UINT16 body_length = (UINT16)length - RDP_PACKET_HEADER_MAX_LENGTH;
 		const UINT16 pad = 8 - (body_length % 8);
@@ -708,7 +716,7 @@ BOOL rdp_write_header(rdpRdp* rdp, wStream* s, size_t length, UINT16 channelId)
 	return TRUE;
 }
 
-static BOOL rdp_security_stream_out(rdpRdp* rdp, wStream* s, size_t length, UINT32 sec_flags,
+static BOOL rdp_security_stream_out(rdpRdp* rdp, wStream* s, size_t length, UINT16 sec_flags,
                                     UINT32* pad)
 {
 	BOOL status = 0;
@@ -716,27 +724,22 @@ static BOOL rdp_security_stream_out(rdpRdp* rdp, wStream* s, size_t length, UINT
 	if (length > UINT16_MAX)
 		return FALSE;
 
-	sec_flags |= rdp->sec_flags;
 	*pad = 0;
 
 	if (sec_flags != 0)
 	{
 		WINPR_ASSERT(sec_flags <= UINT16_MAX);
-		if (!rdp_write_security_header(rdp, s, (UINT16)sec_flags))
+		if (!rdp_write_security_header(rdp, s, sec_flags))
 			return FALSE;
 
 		if (sec_flags & SEC_ENCRYPT)
 		{
-			BOOL res = FALSE;
-			if (!security_lock(rdp))
-				return FALSE;
-
 			if (rdp->settings->EncryptionMethods == ENCRYPTION_METHOD_FIPS)
 			{
 				BYTE* data = Stream_PointerAs(s, BYTE) + 12;
 				const size_t size = WINPR_ASSERTING_INT_CAST(size_t, (data - Stream_Buffer(s)));
 				if (size > length)
-					goto unlock;
+					return FALSE;
 
 				length -= size;
 
@@ -754,24 +757,24 @@ static BOOL rdp_security_stream_out(rdpRdp* rdp, wStream* s, size_t length, UINT
 				Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, *pad));
 
 				if (!Stream_CheckAndLogRequiredCapacityWLog(rdp->log, s, 8))
-					goto unlock;
+					return FALSE;
 				if (!security_hmac_signature(data, length, Stream_Pointer(s), 8, rdp))
-					goto unlock;
+					return FALSE;
 
 				Stream_Seek(s, 8);
 				if (!security_fips_encrypt(data, length + *pad, rdp))
-					goto unlock;
+					return FALSE;
 			}
 			else
 			{
 				const BYTE* data = Stream_PointerAs(s, const BYTE) + 8;
 				const size_t diff = Stream_GetPosition(s) + 8ULL;
 				if (diff > length)
-					goto unlock;
+					return FALSE;
 				length -= diff;
 
 				if (!Stream_CheckAndLogRequiredCapacityWLog(rdp->log, s, 8))
-					goto unlock;
+					return FALSE;
 				if (sec_flags & SEC_SECURE_CHECKSUM)
 					status = security_salted_mac_signature(rdp, data, (UINT32)length, TRUE,
 					                                       Stream_Pointer(s), 8);
@@ -780,24 +783,14 @@ static BOOL rdp_security_stream_out(rdpRdp* rdp, wStream* s, size_t length, UINT
 					                                Stream_PointerAs(s, BYTE), 8);
 
 				if (!status)
-					goto unlock;
+					return FALSE;
 
 				Stream_Seek(s, 8);
 
 				if (!security_encrypt(Stream_Pointer(s), length, rdp))
-					goto unlock;
+					return FALSE;
 			}
-			res = TRUE;
-
-		unlock:
-
-			if (!security_unlock(rdp))
-				return FALSE;
-			if (!res)
-				return FALSE;
 		}
-
-		rdp->sec_flags = 0;
 	}
 
 	return TRUE;
@@ -807,14 +800,14 @@ static UINT32 rdp_get_sec_bytes(rdpRdp* rdp, UINT16 sec_flags)
 {
 	UINT32 sec_bytes = 0;
 
-	if (rdp->sec_flags & SEC_ENCRYPT)
+	if (sec_flags & SEC_ENCRYPT)
 	{
 		sec_bytes = 12;
 
 		if (rdp->settings->EncryptionMethods == ENCRYPTION_METHOD_FIPS)
 			sec_bytes += 4;
 	}
-	else if (rdp->sec_flags != 0 || sec_flags != 0)
+	else if (sec_flags != 0)
 	{
 		sec_bytes = 4;
 	}
@@ -833,10 +826,11 @@ static UINT32 rdp_get_sec_bytes(rdpRdp* rdp, UINT16 sec_flags)
  * @param channel_id channel id
  */
 
-BOOL rdp_send(rdpRdp* rdp, wStream* s, UINT16 channel_id)
+BOOL rdp_send(rdpRdp* rdp, wStream* s, UINT16 channel_id, UINT16 sec_flags)
 {
 	BOOL rc = FALSE;
 	UINT32 pad = 0;
+	BOOL should_unlock = FALSE;
 
 	if (!s)
 		return FALSE;
@@ -844,66 +838,45 @@ BOOL rdp_send(rdpRdp* rdp, wStream* s, UINT16 channel_id)
 	if (!rdp)
 		goto fail;
 
-	size_t length = Stream_GetPosition(s);
-	Stream_SetPosition(s, 0);
-	if (!rdp_write_header(rdp, s, length, channel_id))
-		goto fail;
+	if (sec_flags & SEC_ENCRYPT)
+	{
+		if (!security_lock(rdp))
+			goto fail;
+		should_unlock = TRUE;
+	}
 
-	if (!rdp_security_stream_out(rdp, s, length, 0, &pad))
-		goto fail;
+	{
+		size_t length = Stream_GetPosition(s);
+		Stream_SetPosition(s, 0);
+		if (!rdp_write_header(rdp, s, length, channel_id, sec_flags))
+			goto fail;
 
-	length += pad;
-	Stream_SetPosition(s, length);
-	Stream_SealLength(s);
+		if (!rdp_security_stream_out(rdp, s, length, sec_flags, &pad))
+			goto fail;
+
+		length += pad;
+		Stream_SetPosition(s, length);
+		Stream_SealLength(s);
+	}
 
 	if (transport_write(rdp->transport, s) < 0)
 		goto fail;
 
 	rc = TRUE;
 fail:
+	if (should_unlock && !security_unlock(rdp))
+		rc = FALSE;
 	Stream_Release(s);
 	return rc;
 }
 
-BOOL rdp_send_pdu(rdpRdp* rdp, wStream* s, UINT16 type, UINT16 channel_id)
-{
-	UINT32 sec_bytes = 0;
-	size_t sec_hold = 0;
-	UINT32 pad = 0;
-
-	if (!rdp || !s)
-		return FALSE;
-
-	size_t length = Stream_GetPosition(s);
-	Stream_SetPosition(s, 0);
-	if (!rdp_write_header(rdp, s, length, MCS_GLOBAL_CHANNEL_ID))
-		return FALSE;
-	sec_bytes = rdp_get_sec_bytes(rdp, 0);
-	sec_hold = Stream_GetPosition(s);
-	Stream_Seek(s, sec_bytes);
-	if (!rdp_write_share_control_header(rdp, s, length - sec_bytes, type, channel_id))
-		return FALSE;
-	Stream_SetPosition(s, sec_hold);
-
-	if (!rdp_security_stream_out(rdp, s, length, 0, &pad))
-		return FALSE;
-
-	length += pad;
-	Stream_SetPosition(s, length);
-	Stream_SealLength(s);
-
-	if (transport_write(rdp->transport, s) < 0)
-		return FALSE;
-
-	return TRUE;
-}
-
-BOOL rdp_send_data_pdu(rdpRdp* rdp, wStream* s, BYTE type, UINT16 channel_id)
+BOOL rdp_send_pdu(rdpRdp* rdp, wStream* s, UINT16 type, UINT16 channel_id, UINT16 sec_flags)
 {
 	BOOL rc = FALSE;
 	UINT32 sec_bytes = 0;
 	size_t sec_hold = 0;
 	UINT32 pad = 0;
+	BOOL should_unlock = FALSE;
 
 	if (!s)
 		return FALSE;
@@ -911,25 +884,85 @@ BOOL rdp_send_data_pdu(rdpRdp* rdp, wStream* s, BYTE type, UINT16 channel_id)
 	if (!rdp)
 		goto fail;
 
-	size_t length = Stream_GetPosition(s);
-	Stream_SetPosition(s, 0);
-	if (!rdp_write_header(rdp, s, length, MCS_GLOBAL_CHANNEL_ID))
-		goto fail;
-	sec_bytes = rdp_get_sec_bytes(rdp, 0);
-	sec_hold = Stream_GetPosition(s);
-	Stream_Seek(s, sec_bytes);
-	if (!rdp_write_share_control_header(rdp, s, length - sec_bytes, PDU_TYPE_DATA, channel_id))
-		goto fail;
-	if (!rdp_write_share_data_header(rdp, s, length - sec_bytes, type, rdp->settings->ShareId))
-		goto fail;
-	Stream_SetPosition(s, sec_hold);
+	if (sec_flags & SEC_ENCRYPT)
+	{
+		if (!security_lock(rdp))
+			goto fail;
+		should_unlock = TRUE;
+	}
 
-	if (!rdp_security_stream_out(rdp, s, length, 0, &pad))
+	{
+		size_t length = Stream_GetPosition(s);
+		Stream_SetPosition(s, 0);
+		if (!rdp_write_header(rdp, s, length, MCS_GLOBAL_CHANNEL_ID, sec_flags))
+			goto fail;
+		sec_bytes = rdp_get_sec_bytes(rdp, sec_flags);
+		sec_hold = Stream_GetPosition(s);
+		Stream_Seek(s, sec_bytes);
+		if (!rdp_write_share_control_header(rdp, s, length - sec_bytes, type, channel_id))
+			goto fail;
+		Stream_SetPosition(s, sec_hold);
+
+		if (!rdp_security_stream_out(rdp, s, length, sec_flags, &pad))
+			goto fail;
+
+		length += pad;
+		Stream_SetPosition(s, length);
+		Stream_SealLength(s);
+	}
+
+	if (transport_write(rdp->transport, s) < 0)
 		goto fail;
 
-	length += pad;
-	Stream_SetPosition(s, length);
-	Stream_SealLength(s);
+	rc = TRUE;
+fail:
+	if (should_unlock && !security_unlock(rdp))
+		rc = FALSE;
+	return rc;
+}
+
+BOOL rdp_send_data_pdu(rdpRdp* rdp, wStream* s, BYTE type, UINT16 channel_id, UINT16 sec_flags)
+{
+	BOOL rc = FALSE;
+	UINT32 sec_bytes = 0;
+	size_t sec_hold = 0;
+	UINT32 pad = 0;
+	BOOL should_unlock = FALSE;
+
+	if (!s)
+		return FALSE;
+
+	if (!rdp)
+		goto fail;
+
+	if (sec_flags & SEC_ENCRYPT)
+	{
+		if (!security_lock(rdp))
+			goto fail;
+		should_unlock = TRUE;
+	}
+
+	{
+		size_t length = Stream_GetPosition(s);
+		Stream_SetPosition(s, 0);
+		if (!rdp_write_header(rdp, s, length, MCS_GLOBAL_CHANNEL_ID, sec_flags))
+			goto fail;
+		sec_bytes = rdp_get_sec_bytes(rdp, sec_flags);
+		sec_hold = Stream_GetPosition(s);
+		Stream_Seek(s, sec_bytes);
+		if (!rdp_write_share_control_header(rdp, s, length - sec_bytes, PDU_TYPE_DATA, channel_id))
+			goto fail;
+		if (!rdp_write_share_data_header(rdp, s, length - sec_bytes, type, rdp->settings->ShareId))
+			goto fail;
+		Stream_SetPosition(s, sec_hold);
+
+		if (!rdp_security_stream_out(rdp, s, length, sec_flags, &pad))
+			goto fail;
+
+		length += pad;
+		Stream_SetPosition(s, length);
+		Stream_SealLength(s);
+	}
 	WLog_Print(rdp->log, WLOG_DEBUG,
 	           "sending data (type=0x%x size=%" PRIuz " channelId=%" PRIu16 ")", type,
 	           Stream_Length(s), channel_id);
@@ -940,6 +973,8 @@ BOOL rdp_send_data_pdu(rdpRdp* rdp, wStream* s, BYTE type, UINT16 channel_id)
 
 	rc = TRUE;
 fail:
+	if (should_unlock && !security_unlock(rdp))
+		rc = FALSE;
 	Stream_Release(s);
 	return rc;
 }
@@ -948,20 +983,30 @@ BOOL rdp_send_message_channel_pdu(rdpRdp* rdp, wStream* s, UINT16 sec_flags)
 {
 	BOOL rc = FALSE;
 	UINT32 pad = 0;
+	BOOL should_unlock = FALSE;
 
 	WINPR_ASSERT(rdp);
 	WINPR_ASSERT(s);
 
-	size_t length = Stream_GetPosition(s);
-	Stream_SetPosition(s, 0);
-	if (!rdp_write_header(rdp, s, length, rdp->mcs->messageChannelId))
-		goto fail;
+	if (sec_flags & SEC_ENCRYPT)
+	{
+		if (!security_lock(rdp))
+			goto fail;
+		should_unlock = TRUE;
+	}
 
-	if (!rdp_security_stream_out(rdp, s, length, sec_flags, &pad))
-		goto fail;
+	{
+		size_t length = Stream_GetPosition(s);
+		Stream_SetPosition(s, 0);
+		if (!rdp_write_header(rdp, s, length, rdp->mcs->messageChannelId, sec_flags))
+			goto fail;
 
-	length += pad;
-	Stream_SetPosition(s, length);
+		if (!rdp_security_stream_out(rdp, s, length, sec_flags, &pad))
+			goto fail;
+
+		length += pad;
+		Stream_SetPosition(s, length);
+	}
 	Stream_SealLength(s);
 
 	if (transport_write(rdp->transport, s) < 0)
@@ -969,6 +1014,8 @@ BOOL rdp_send_message_channel_pdu(rdpRdp* rdp, wStream* s, UINT16 sec_flags)
 
 	rc = TRUE;
 fail:
+	if (should_unlock && !security_unlock(rdp))
+		rc = FALSE;
 	Stream_Release(s);
 	return rc;
 }
@@ -996,7 +1043,7 @@ static BOOL rdp_recv_server_set_keyboard_indicators_pdu(rdpRdp* rdp, wStream* s)
 	{
 		WLog_Print(rdp->log, WLOG_WARN,
 		           "[MS-RDPBCGR] 2.2.8.2.1.1 Set Keyboard Indicators PDU Data "
-		           "(TS_SET_KEYBOARD_INDICATORS_PDU)::unitId should be 0, is %" PRIu8,
+		           "(TS_SET_KEYBOARD_INDICATORS_PDU)::unitId should be 0, is %" PRIu16,
 		           unitId);
 	}
 	const UINT16 ledFlags = Stream_Get_UINT16(s); /* ledFlags (2 bytes) */
@@ -1016,7 +1063,7 @@ static BOOL rdp_recv_server_set_keyboard_ime_status_pdu(rdpRdp* rdp, wStream* s)
 	{
 		WLog_Print(rdp->log, WLOG_WARN,
 		           "[MS-RDPBCGR] 2.2.8.2.2.1 Set Keyboard IME Status PDU Data "
-		           "(TS_SET_KEYBOARD_IME_STATUS_PDU)::unitId should be 0, is %" PRIu8,
+		           "(TS_SET_KEYBOARD_IME_STATUS_PDU)::unitId should be 0, is %" PRIu16,
 		           unitId);
 	}
 	const uint32_t imeState = Stream_Get_UINT32(s);    /* imeState (4 bytes) */
@@ -1622,13 +1669,12 @@ static state_run_t rdp_recv_tpkt_pdu(rdpRdp* rdp, wStream* s)
 	UINT16 pduSource = 0;
 	UINT16 channelId = 0;
 	UINT16 securityFlags = 0;
-	freerdp* instance = NULL;
 
 	WINPR_ASSERT(rdp);
 	WINPR_ASSERT(rdp->context);
 	WINPR_ASSERT(s);
 
-	instance = rdp->context->instance;
+	freerdp* instance = rdp->context->instance;
 	WINPR_ASSERT(instance);
 
 	if (!rdp_read_header(rdp, s, &length, &channelId))
@@ -1736,7 +1782,7 @@ static state_run_t rdp_recv_tpkt_pdu(rdpRdp* rdp, wStream* s)
 			{
 				char buffer[256] = { 0 };
 				WLog_Print(rdp->log, WLOG_WARN,
-				           "pduType %s not properly parsed, %" PRIdz
+				           "pduType %s not properly parsed, %" PRIuz
 				           " bytes remaining unhandled. Skipping.",
 				           pdu_type_to_str(pduType, buffer, sizeof(buffer)), diff);
 			}
@@ -1834,7 +1880,7 @@ static state_run_t rdp_client_exchange_monitor_layout(rdpRdp* rdp, wStream* s)
 	WINPR_ASSERT(rdp);
 
 	if (!rdp_check_monitor_layout_pdu_state(rdp, FALSE))
-		return FALSE;
+		return STATE_RUN_FAILED;
 
 	/* We might receive unrelated messages from the server (channel traffic),
 	 * so only proceed if some flag changed
@@ -1893,7 +1939,7 @@ static state_run_t rdp_recv_callback_int(WINPR_ATTR_UNUSED rdpTransport* transpo
 				nego_recv(rdp->transport, s, (void*)rdp->nego);
 
 				if (!nego_update_settings_from_state(rdp->nego, rdp->settings))
-					return FALSE;
+					return STATE_RUN_FAILED;
 
 				if (nego_get_state(rdp->nego) != NEGO_STATE_FINAL)
 				{
@@ -2043,7 +2089,7 @@ static state_run_t rdp_recv_callback_int(WINPR_ATTR_UNUSED rdpTransport* transpo
 			break;
 
 		case CONNECTION_STATE_CONNECT_TIME_AUTO_DETECT_REQUEST:
-			if (!rdp_client_connect_auto_detect(rdp, s))
+			if (!rdp_client_connect_auto_detect(rdp, s, WLOG_DEBUG))
 			{
 				if (!rdp_client_transition_to_state(rdp, CONNECTION_STATE_LICENSING))
 					status = STATE_RUN_FAILED;
@@ -2066,7 +2112,7 @@ static state_run_t rdp_recv_callback_int(WINPR_ATTR_UNUSED rdpTransport* transpo
 			break;
 
 		case CONNECTION_STATE_MULTITRANSPORT_BOOTSTRAPPING_REQUEST:
-			if (!rdp_client_connect_auto_detect(rdp, s))
+			if (!rdp_client_connect_auto_detect(rdp, s, WLOG_DEBUG))
 			{
 				(void)rdp_client_transition_to_state(
 				    rdp, CONNECTION_STATE_CAPABILITIES_EXCHANGE_DEMAND_ACTIVE);
@@ -2133,7 +2179,7 @@ static state_run_t rdp_recv_callback_int(WINPR_ATTR_UNUSED rdpTransport* transpo
 			break;
 
 		default:
-			WLog_Print(rdp->log, WLOG_ERROR, "%s state %d", rdp_get_state_string(rdp),
+			WLog_Print(rdp->log, WLOG_ERROR, "%s state %u", rdp_get_state_string(rdp),
 			           rdp_get_state(rdp));
 			status = STATE_RUN_FAILED;
 			break;
@@ -2192,19 +2238,20 @@ BOOL rdp_channel_send_packet(rdpRdp* rdp, UINT16 channelId, size_t totalSize, UI
 
 BOOL rdp_send_error_info(rdpRdp* rdp)
 {
+	UINT16 sec_flags = 0;
 	wStream* s = NULL;
 	BOOL status = 0;
 
 	if (rdp->errorInfo == ERRINFO_SUCCESS)
 		return TRUE;
 
-	s = rdp_data_pdu_init(rdp);
+	s = rdp_data_pdu_init(rdp, &sec_flags);
 
 	if (!s)
 		return FALSE;
 
 	Stream_Write_UINT32(s, rdp->errorInfo); /* error id (4 bytes) */
-	status = rdp_send_data_pdu(rdp, s, DATA_PDU_TYPE_SET_ERROR_INFO, 0);
+	status = rdp_send_data_pdu(rdp, s, DATA_PDU_TYPE_SET_ERROR_INFO, 0, sec_flags);
 	return status;
 }
 
@@ -2240,11 +2287,13 @@ int rdp_check_fds(rdpRdp* rdp)
 
 	if (status < 0)
 		WLog_Print(rdp->log, WLOG_DEBUG, "transport_check_fds() - %i", status);
+	else
+		status = freerdp_timer_poll(rdp->timer);
 
 	return status;
 }
 
-BOOL freerdp_get_stats(rdpRdp* rdp, UINT64* inBytes, UINT64* outBytes, UINT64* inPackets,
+BOOL freerdp_get_stats(const rdpRdp* rdp, UINT64* inBytes, UINT64* outBytes, UINT64* inPackets,
                        UINT64* outPackets)
 {
 	if (!rdp)
@@ -2262,6 +2311,46 @@ BOOL freerdp_get_stats(rdpRdp* rdp, UINT64* inBytes, UINT64* outBytes, UINT64* i
 	return TRUE;
 }
 
+static bool rdp_new_common(rdpRdp* rdp)
+{
+	WINPR_ASSERT(rdp);
+
+	bool rc = false;
+	rdp->transport = transport_new(rdp->context);
+	if (!rdp->transport)
+		goto fail;
+
+	if (rdp->io)
+	{
+		if (!transport_set_io_callbacks(rdp->transport, rdp->io))
+			goto fail;
+	}
+
+	rdp->aad = aad_new(rdp->context);
+	if (!rdp->aad)
+		goto fail;
+
+	rdp->nego = nego_new(rdp->transport);
+	if (!rdp->nego)
+		goto fail;
+
+	rdp->mcs = mcs_new(rdp->context);
+	if (!rdp->mcs)
+		goto fail;
+
+	rdp->license = license_new(rdp);
+	if (!rdp->license)
+		goto fail;
+
+	rdp->fastpath = fastpath_new(rdp);
+	if (!rdp->fastpath)
+		goto fail;
+
+	rc = true;
+fail:
+	return rc;
+}
+
 /**
  * Instantiate new RDP module.
  * @return new RDP module
@@ -2269,9 +2358,8 @@ BOOL freerdp_get_stats(rdpRdp* rdp, UINT64* inBytes, UINT64* outBytes, UINT64* i
 
 rdpRdp* rdp_new(rdpContext* context)
 {
-	rdpRdp* rdp = NULL;
 	DWORD flags = 0;
-	rdp = (rdpRdp*)calloc(1, sizeof(rdpRdp));
+	rdpRdp* rdp = (rdpRdp*)calloc(1, sizeof(rdpRdp));
 
 	if (!rdp)
 		return NULL;
@@ -2317,9 +2405,7 @@ rdpRdp* rdp_new(rdpContext* context)
 #endif
 	}
 
-	rdp->transport = transport_new(context);
-
-	if (!rdp->transport)
+	if (!rdp_new_common(rdp))
 		goto fail;
 
 	{
@@ -2332,15 +2418,6 @@ rdpRdp* rdp_new(rdpContext* context)
 		*rdp->io = *io;
 	}
 
-	rdp->aad = aad_new(context, rdp->transport);
-	if (!rdp->aad)
-		goto fail;
-
-	rdp->license = license_new(rdp);
-
-	if (!rdp->license)
-		goto fail;
-
 	rdp->input = input_new(rdp);
 
 	if (!rdp->input)
@@ -2351,21 +2428,6 @@ rdpRdp* rdp_new(rdpContext* context)
 	if (!rdp->update)
 		goto fail;
 
-	rdp->fastpath = fastpath_new(rdp);
-
-	if (!rdp->fastpath)
-		goto fail;
-
-	rdp->nego = nego_new(rdp->transport);
-
-	if (!rdp->nego)
-		goto fail;
-
-	rdp->mcs = mcs_new(rdp->transport);
-
-	if (!rdp->mcs)
-		goto fail;
-
 	rdp->redirection = redirection_new();
 
 	if (!rdp->redirection)
@@ -2399,6 +2461,11 @@ rdpRdp* rdp_new(rdpContext* context)
 	rdp->abortEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
 	if (!rdp->abortEvent)
 		goto fail;
+
+	rdp->timer = freerdp_timer_new(rdp);
+	if (!rdp->timer)
+		goto fail;
+
 	return rdp;
 
 fail:
@@ -2423,12 +2490,14 @@ static void rdp_reset_free(rdpRdp* rdp)
 	rdp->fips_decrypt = NULL;
 	(void)security_unlock(rdp);
 
+	aad_free(rdp->aad);
 	mcs_free(rdp->mcs);
 	nego_free(rdp->nego);
 	license_free(rdp->license);
 	transport_free(rdp->transport);
 	fastpath_free(rdp->fastpath);
 
+	rdp->aad = NULL;
 	rdp->mcs = NULL;
 	rdp->nego = NULL;
 	rdp->license = NULL;
@@ -2439,15 +2508,10 @@ static void rdp_reset_free(rdpRdp* rdp)
 BOOL rdp_reset(rdpRdp* rdp)
 {
 	BOOL rc = TRUE;
-	rdpContext* context = NULL;
-	rdpSettings* settings = NULL;
 
 	WINPR_ASSERT(rdp);
 
-	context = rdp->context;
-	WINPR_ASSERT(context);
-
-	settings = rdp->settings;
+	rdpSettings* settings = rdp->settings;
 	WINPR_ASSERT(settings);
 
 	bulk_reset(rdp->bulk);
@@ -2466,41 +2530,13 @@ BOOL rdp_reset(rdpRdp* rdp)
 	if (!rc)
 		goto fail;
 
-	rc = FALSE;
-	rdp->transport = transport_new(context);
-	if (!rdp->transport)
-		goto fail;
-
-	if (rdp->io)
-	{
-		if (!transport_set_io_callbacks(rdp->transport, rdp->io))
-			goto fail;
-	}
-
-	aad_free(rdp->aad);
-	rdp->aad = aad_new(context, rdp->transport);
-	if (!rdp->aad)
-		goto fail;
-
-	rdp->nego = nego_new(rdp->transport);
-	if (!rdp->nego)
-		goto fail;
-
-	rdp->mcs = mcs_new(rdp->transport);
-	if (!rdp->mcs)
+	rc = rdp_new_common(rdp);
+	if (!rc)
 		goto fail;
 
 	if (!transport_set_layer(rdp->transport, TRANSPORT_LAYER_TCP))
 		goto fail;
 
-	rdp->license = license_new(rdp);
-	if (!rdp->license)
-		goto fail;
-
-	rdp->fastpath = fastpath_new(rdp);
-	if (!rdp->fastpath)
-		goto fail;
-
 	rdp->errorInfo = 0;
 	rc = rdp_finalize_reset_flags(rdp, TRUE);
 
@@ -2517,6 +2553,7 @@ void rdp_free(rdpRdp* rdp)
 {
 	if (rdp)
 	{
+		freerdp_timer_free(rdp->timer);
 		rdp_reset_free(rdp);
 
 		freerdp_settings_free(rdp->settings);
@@ -2934,7 +2971,7 @@ static void log_build_warn(rdpRdp* rdp, const char* what, const char* msg,
 				WLog_Print(rdp->log, WLOG_WARN, "* '%s'", tok);
 				tok = strtok_s(NULL, " ", &saveptr);
 			}
-			WLog_Print(rdp->log, WLOG_WARN, "");
+			WLog_Print(rdp->log, WLOG_WARN, "*");
 			WLog_Print(rdp->log, WLOG_WARN, "[%s] build options %s", what, msg);
 			WLog_Print(rdp->log, WLOG_WARN, "*************************************************");
 		}
@@ -2953,10 +2990,10 @@ static void print_first_line_int(wLog* log, log_line_t* firstLine, const char* w
 		const DWORD level = WLOG_WARN;
 		if (WLog_IsLevelActive(log, level))
 		{
-			WLog_PrintMessage(log, WLOG_MESSAGE_TEXT, level, line, file, fkt,
-			                  "*************************************************");
-			WLog_PrintMessage(log, WLOG_MESSAGE_TEXT, level, line, file, fkt,
-			                  "[SSL] {%s} build or configuration missing:", what);
+			WLog_PrintTextMessage(log, level, line, file, fkt,
+			                      "*************************************************");
+			WLog_PrintTextMessage(log, level, line, file, fkt,
+			                      "[SSL] {%s} build or configuration missing:", what);
 		}
 		firstLine->line = line;
 		firstLine->file = file;
@@ -2971,9 +3008,9 @@ static void print_last_line(wLog* log, const log_line_t* firstLine)
 	if (firstLine->fkt)
 	{
 		if (WLog_IsLevelActive(log, firstLine->level))
-			WLog_PrintMessage(log, WLOG_MESSAGE_TEXT, firstLine->level, firstLine->line,
-			                  firstLine->file, firstLine->fkt,
-			                  "*************************************************");
+			WLog_PrintTextMessage(log, firstLine->level, firstLine->line, firstLine->file,
+			                      firstLine->fkt,
+			                      "*************************************************");
 	}
 }
 
@@ -3108,3 +3145,18 @@ void rdp_log_build_warnings(rdpRdp* rdp)
 	               option_is_runtime_checks);
 	log_build_warn_ssl(rdp);
 }
+
+size_t rdp_get_event_handles(rdpRdp* rdp, HANDLE* handles, uint32_t count)
+{
+	size_t nCount = transport_get_event_handles(rdp->transport, handles, count);
+
+	if (nCount == 0)
+		return 0;
+
+	if (count < nCount + 2UL)
+		return 0;
+
+	handles[nCount++] = utils_get_abort_event(rdp);
+	handles[nCount++] = freerdp_timer_get_event(rdp->timer);
+	return nCount;
+}
