@@ -93,7 +93,7 @@ static void tls_print_certificate_name_mismatch_error(const char* hostname, UINT
                                                       const char* common_name, char** alt_names,
                                                       size_t alt_names_count);
 static void tls_print_new_certificate_warn(rdpCertificateStore* store, const char* hostname,
-                                           UINT16 port, const char* fingerprint);
+                                           UINT16 port, const char* type, const char* fingerprint);
 static void tls_print_certificate_error(rdpCertificateStore* store, rdpCertificateData* stored_data,
                                         const char* hostname, UINT16 port, const char* fingerprint);
 
@@ -251,7 +251,7 @@ static int bio_rdp_tls_puts(BIO* bio, const char* str)
 	if (!str)
 		return 0;
 
-	const int max = (INT_MAX > SIZE_MAX) ? SIZE_MAX : INT_MAX;
+	const size_t max = (INT_MAX > SIZE_MAX) ? SIZE_MAX : INT_MAX;
 	const size_t size = strnlen(str, max);
 	if (size >= max)
 		return -1;
@@ -363,7 +363,7 @@ static long bio_rdp_tls_ctrl(BIO* bio, int cmd, long num, void* ptr)
 			BIO_clear_retry_flags(bio);
 			status = BIO_ctrl(ssl_wbio, cmd, num, ptr);
 			if (status != 1)
-				WLog_DBG(TAG, "BIO_ctrl returned %d", status);
+				WLog_DBG(TAG, "BIO_ctrl returned %ld", status);
 			BIO_copy_next_retry(bio);
 			status = 1;
 			break;
@@ -683,11 +683,13 @@ static SecPkgContext_Bindings* tls_get_channel_bindings(const rdpCertificate* ce
 	if (!ContextBindings)
 		goto out_free;
 
-	const size_t slen = sizeof(SEC_CHANNEL_BINDINGS) + ChannelBindingTokenLength;
-	if (slen > UINT32_MAX)
-		goto out_free;
+	{
+		const size_t slen = sizeof(SEC_CHANNEL_BINDINGS) + ChannelBindingTokenLength;
+		if (slen > UINT32_MAX)
+			goto out_free;
 
-	ContextBindings->BindingsLength = (UINT32)slen;
+		ContextBindings->BindingsLength = (UINT32)slen;
+	}
 	ChannelBindings = (SEC_CHANNEL_BINDINGS*)calloc(1, ContextBindings->BindingsLength);
 
 	if (!ChannelBindings)
@@ -793,13 +795,13 @@ static BOOL tls_prepare(rdpTls* tls, BIO* underlying, SSL_METHOD* method, int op
 	UINT16 version = freerdp_settings_get_uint16(settings, FreeRDP_TLSMinVersion);
 	if (!SSL_CTX_set_min_proto_version(tls->ctx, version))
 	{
-		WLog_ERR(TAG, "SSL_CTX_set_min_proto_version %s failed", version);
+		WLog_ERR(TAG, "SSL_CTX_set_min_proto_version %" PRIu16 " failed", version);
 		return FALSE;
 	}
 	version = freerdp_settings_get_uint16(settings, FreeRDP_TLSMaxVersion);
 	if (!SSL_CTX_set_max_proto_version(tls->ctx, version))
 	{
-		WLog_ERR(TAG, "SSL_CTX_set_max_proto_version %s failed", version);
+		WLog_ERR(TAG, "SSL_CTX_set_max_proto_version %" PRIu16 " failed", version);
 		return FALSE;
 	}
 #endif
@@ -912,7 +914,7 @@ TlsHandshakeResult freerdp_tls_connect_ex(rdpTls* tls, BIO* underlying, const SS
 	adjustSslOptions(&options);
 
 	if (!tls_prepare(tls, underlying, methods, options, TRUE))
-		return 0;
+		return TLS_HANDSHAKE_ERROR;
 
 #if !defined(OPENSSL_NO_TLSEXT)
 	const char* str = tls_get_server_name(tls);
@@ -1448,11 +1450,13 @@ static BOOL is_accepted_fingerprint(const rdpCertificate* cert,
 			if (!h)
 				goto next;
 
-			const char* fp = h + strlen(h) + 1;
-			if (compare_fingerprint_all(fp, h, cert))
 			{
-				rc = TRUE;
-				break;
+				const char* fp = h + strlen(h) + 1;
+				if (compare_fingerprint_all(fp, h, cert))
+				{
+					rc = TRUE;
+					break;
+				}
 			}
 		next:
 			cur = strtok_s(NULL, ",", &context);
@@ -1509,7 +1513,7 @@ static BOOL tls_extract_full_pem(const rdpCertificate* cert, BYTE** PublicKey,
 
 static int tls_config_parse_bool(WINPR_JSON* json, const char* opt)
 {
-	WINPR_JSON* val = WINPR_JSON_GetObjectItem(json, opt);
+	WINPR_JSON* val = WINPR_JSON_GetObjectItemCaseSensitive(json, opt);
 	if (!val || !WINPR_JSON_IsBool(val))
 		return -1;
 
@@ -1525,7 +1529,7 @@ static int tls_config_check_allowed_hashed(const char* configfile, const rdpCert
 	WINPR_ASSERT(cert);
 	WINPR_ASSERT(json);
 
-	WINPR_JSON* db = WINPR_JSON_GetObjectItem(json, "certificate-db");
+	WINPR_JSON* db = WINPR_JSON_GetObjectItemCaseSensitive(json, "certificate-db");
 	if (!db || !WINPR_JSON_IsArray(db))
 		return 0;
 
@@ -1540,7 +1544,7 @@ static int tls_config_check_allowed_hashed(const char* configfile, const rdpCert
 			continue;
 		}
 
-		WINPR_JSON* key = WINPR_JSON_GetObjectItem(cur, "type");
+		WINPR_JSON* key = WINPR_JSON_GetObjectItemCaseSensitive(cur, "type");
 		if (!key || !WINPR_JSON_IsString(key))
 		{
 			WLog_WARN(TAG,
@@ -1549,7 +1553,7 @@ static int tls_config_check_allowed_hashed(const char* configfile, const rdpCert
 			          configfile, x);
 			continue;
 		}
-		WINPR_JSON* val = WINPR_JSON_GetObjectItem(cur, "hash");
+		WINPR_JSON* val = WINPR_JSON_GetObjectItemCaseSensitive(cur, "hash");
 		if (!val || !WINPR_JSON_IsString(val))
 		{
 			WLog_WARN(TAG,
@@ -1650,12 +1654,17 @@ int tls_verify_certificate(rdpTls* tls, const rdpCertificate* cert, const char*
 	DWORD flags = VERIFY_CERT_FLAG_NONE;
 
 	WINPR_ASSERT(tls);
-	WINPR_ASSERT(tls->context);
 
-	freerdp* instance = tls->context->instance;
+	rdpContext* context = tls->context;
+	WINPR_ASSERT(context);
+
+	freerdp* instance = context->instance;
 	WINPR_ASSERT(instance);
 
-	if (freerdp_shall_disconnect_context(instance->context))
+	const rdpSettings* settings = context->settings;
+	WINPR_ASSERT(settings);
+
+	if (freerdp_shall_disconnect_context(context))
 		return -1;
 
 	if (!tls_extract_full_pem(cert, &pemCert, &length))
@@ -1668,7 +1677,7 @@ int tls_verify_certificate(rdpTls* tls, const rdpCertificate* cert, const char*
 		goto end;
 	}
 
-	if (is_accepted_fingerprint(cert, tls->context->settings->CertificateAcceptedFingerprints))
+	if (is_accepted_fingerprint(cert, settings->CertificateAcceptedFingerprints))
 	{
 		verification_status = 1;
 		goto end;
@@ -1684,7 +1693,7 @@ int tls_verify_certificate(rdpTls* tls, const rdpCertificate* cert, const char*
 		flags |= VERIFY_CERT_FLAG_REDIRECT;
 
 	/* Certificate management is done by the application */
-	if (tls->context->settings->ExternalCertificateManagement)
+	if (settings->ExternalCertificateManagement)
 	{
 		if (instance->VerifyX509Certificate)
 			verification_status =
@@ -1702,7 +1711,7 @@ int tls_verify_certificate(rdpTls* tls, const rdpCertificate* cert, const char*
 		}
 	}
 	/* ignore certificate verification if user explicitly required it (discouraged) */
-	else if (freerdp_settings_get_bool(tls->context->settings, FreeRDP_IgnoreCertificate))
+	else if (freerdp_settings_get_bool(settings, FreeRDP_IgnoreCertificate))
 	{
 		WLog_WARN(TAG, "[DANGER] Certificate not checked, /cert:ignore in use.");
 		WLog_WARN(TAG, "[DANGER] This prevents MITM attacks from being detected!");
@@ -1710,13 +1719,13 @@ int tls_verify_certificate(rdpTls* tls, const rdpCertificate* cert, const char*
 		          "[DANGER] Avoid using this unless in a secure LAN (=no internet) environment");
 		verification_status = 1; /* success! */
 	}
-	else if (!tls->isGatewayTransport && (tls->context->settings->AuthenticationLevel == 0))
+	else if (!tls->isGatewayTransport && (settings->AuthenticationLevel == 0))
 		verification_status = 1; /* success! */
 	else
 	{
 		/* if user explicitly specified a certificate name, use it instead of the hostname */
-		if (!tls->isGatewayTransport && tls->context->settings->CertificateName)
-			hostname = tls->context->settings->CertificateName;
+		if (!tls->isGatewayTransport && settings->CertificateName)
+			hostname = settings->CertificateName;
 
 		/* attempt verification using OpenSSL and the ~/.freerdp/certs certificate store */
 		certificate_status = freerdp_certificate_verify(
@@ -1789,18 +1798,29 @@ int tls_verify_certificate(rdpTls* tls, const rdpCertificate* cert, const char*
 					                                          dns_names, dns_names_count);
 
 				{
+					const char* type = "";
+					if (freerdp_settings_get_bool(settings, FreeRDP_AutoAcceptCertificate))
+						type = "tofo (auto-accept)";
+
+					if (freerdp_settings_get_bool(settings, FreeRDP_AutoDenyCertificate))
+						type = "strict (auto-deny)";
+
+					if (freerdp_settings_get_bool(settings, FreeRDP_IgnoreCertificate))
+						type = "ignore (no check)";
+
 					char* efp = freerdp_certificate_get_fingerprint(cert);
-					tls_print_new_certificate_warn(tls->certificate_store, hostname, port, efp);
+					tls_print_new_certificate_warn(tls->certificate_store, hostname, port, type,
+					                               efp);
 					free(efp);
 				}
 
 				/* Automatically accept certificate on first use */
-				if (tls->context->settings->AutoAcceptCertificate)
+				if (settings->AutoAcceptCertificate)
 				{
 					WLog_INFO(TAG, "No certificate stored, automatically accepting.");
 					accept_certificate = 1;
 				}
-				else if (tls->context->settings->AutoDenyCertificate)
+				else if (settings->AutoDenyCertificate)
 				{
 					WLog_INFO(TAG, "No certificate stored, automatically denying.");
 					accept_certificate = 0;
@@ -1819,8 +1839,8 @@ int tls_verify_certificate(rdpTls* tls, const rdpCertificate* cert, const char*
 				}
 				else if (instance->VerifyCertificateEx)
 				{
-					const BOOL use_pem = freerdp_settings_get_bool(
-					    tls->context->settings, FreeRDP_CertificateCallbackPreferPEM);
+					const BOOL use_pem =
+					    freerdp_settings_get_bool(settings, FreeRDP_CertificateCallbackPreferPEM);
 					char* fp = NULL;
 					DWORD cflags = flags;
 					if (use_pem)
@@ -1865,7 +1885,7 @@ int tls_verify_certificate(rdpTls* tls, const rdpCertificate* cert, const char*
 					WLog_WARN(TAG, "Failed to get certificate entry for %s:%" PRIu16 "", hostname,
 					          port);
 
-				if (tls->context->settings->AutoDenyCertificate)
+				if (settings->AutoDenyCertificate)
 				{
 					WLog_INFO(TAG, "No certificate stored, automatically denying.");
 					accept_certificate = 0;
@@ -1892,8 +1912,7 @@ int tls_verify_certificate(rdpTls* tls, const rdpCertificate* cert, const char*
 					const char* old_pem = freerdp_certificate_data_get_pem(stored_data);
 					const BOOL fpIsAllocated =
 					    !old_pem ||
-					    !freerdp_settings_get_bool(tls->context->settings,
-					                               FreeRDP_CertificateCallbackPreferPEM);
+					    !freerdp_settings_get_bool(settings, FreeRDP_CertificateCallbackPreferPEM);
 					char* fp = NULL;
 					if (!fpIsAllocated)
 					{
@@ -1975,7 +1994,7 @@ end:
 }
 
 void tls_print_new_certificate_warn(rdpCertificateStore* store, const char* hostname, UINT16 port,
-                                    const char* fingerprint)
+                                    const char* type, const char* fingerprint)
 {
 	char* path = freerdp_certificate_store_get_cert_path(store, hostname, port);
 
@@ -1989,7 +2008,8 @@ void tls_print_new_certificate_warn(rdpCertificateStore* store, const char* host
 	WLog_ERR(TAG, "The fingerprint for the host key sent by the remote host is %s", fingerprint);
 	WLog_ERR(TAG, "Please contact your system administrator.");
 	WLog_ERR(TAG, "Add correct host key in %s to get rid of this message.", path);
-	WLog_ERR(TAG, "Host key for %s has changed and you have requested strict checking.", hostname);
+	WLog_ERR(TAG, "Host key for %s has changed and you have requested %s checking.", hostname,
+	         type);
 	WLog_ERR(TAG, "Host key verification failed.");
 
 	free(path);
