@@ -31,6 +31,7 @@
 #include <freerdp/api.h>
 #include <freerdp/log.h>
 #include <freerdp/freerdp.h>
+#include <freerdp/codecs.h>
 
 #include <freerdp/gdi/gdi.h>
 #include <freerdp/gdi/dc.h>
@@ -487,8 +488,10 @@ BOOL gdi_bitmap_update(rdpContext* context, const BITMAP_UPDATE* bitmapUpdate)
 		WLog_ERR(TAG,
 		         "Invalid arguments: context=%p, bitmapUpdate=%p, context->gdi=%p, "
 		         "context->codecs=%p",
-		         context, bitmapUpdate, context ? context->gdi : NULL,
-		         context ? context->codecs : NULL);
+		         WINPR_CXX_COMPAT_CAST(const void*, context),
+		         WINPR_CXX_COMPAT_CAST(const void*, bitmapUpdate),
+		         WINPR_CXX_COMPAT_CAST(const void*, context ? context->gdi : NULL),
+		         WINPR_CXX_COMPAT_CAST(const void*, context ? context->codecs : NULL));
 		return FALSE;
 	}
 
@@ -716,7 +719,6 @@ static BOOL gdi_opaque_rect(rdpContext* context, const OPAQUE_RECT_ORDER* opaque
 	INT32 y = opaque_rect->nTopRect;
 	INT32 w = opaque_rect->nWidth;
 	INT32 h = opaque_rect->nHeight;
-	gdi_ClipCoords(gdi->drawing->hdc, &x, &y, &w, &h, NULL, NULL);
 	gdi_CRgnToRect(x, y, w, h, &rect);
 
 	if (!gdi_decode_color(gdi, opaque_rect->color, &brush_color, NULL))
@@ -750,11 +752,11 @@ static BOOL gdi_multi_opaque_rect(rdpContext* context,
 	for (UINT32 i = 0; i < multi_opaque_rect->numRectangles; i++)
 	{
 		const DELTA_RECT* rectangle = &multi_opaque_rect->rectangles[i];
-		INT32 x = rectangle->left;
-		INT32 y = rectangle->top;
-		INT32 w = rectangle->width;
-		INT32 h = rectangle->height;
-		gdi_ClipCoords(gdi->drawing->hdc, &x, &y, &w, &h, NULL, NULL);
+		const INT32 x = rectangle->left;
+		const INT32 y = rectangle->top;
+		const INT32 w = rectangle->width;
+		const INT32 h = rectangle->height;
+
 		gdi_CRgnToRect(x, y, w, h, &rect);
 		ret = gdi_FillRect(gdi->drawing->hdc, &rect, hBrush);
 
@@ -769,24 +771,22 @@ static BOOL gdi_multi_opaque_rect(rdpContext* context,
 static BOOL gdi_line_to(rdpContext* context, const LINE_TO_ORDER* lineTo)
 {
 	UINT32 color = 0;
-	HGDI_PEN hPen = NULL;
+	WINPR_ASSERT(context);
+	WINPR_ASSERT(lineTo);
+
 	rdpGdi* gdi = context->gdi;
-	INT32 xStart = lineTo->nXStart;
-	INT32 yStart = lineTo->nYStart;
-	INT32 xEnd = lineTo->nXEnd;
-	INT32 yEnd = lineTo->nYEnd;
-	INT32 w = 0;
-	INT32 h = 0;
-	gdi_ClipCoords(gdi->drawing->hdc, &xStart, &yStart, &w, &h, NULL, NULL);
-	gdi_ClipCoords(gdi->drawing->hdc, &xEnd, &yEnd, &w, &h, NULL, NULL);
+	WINPR_ASSERT(gdi);
 
 	if (!gdi_decode_color(gdi, lineTo->penColor, &color, NULL))
 		return FALSE;
 
-	if (!(hPen = gdi_CreatePen(lineTo->penStyle, lineTo->penWidth, color, gdi->drawing->hdc->format,
-	                           &gdi->palette)))
+	HGDI_PEN hPen = gdi_CreatePen(lineTo->penStyle, lineTo->penWidth, color,
+	                              gdi->drawing->hdc->format, &gdi->palette);
+	if (!hPen)
 		return FALSE;
 
+	WINPR_ASSERT(gdi->drawing);
+
 	gdi_SelectObject(gdi->drawing->hdc, (HGDIOBJECT)hPen);
 	gdi_SetROP2(gdi->drawing->hdc, WINPR_ASSERTING_INT_CAST(int32_t, lineTo->bRop2));
 	gdi_MoveToEx(gdi->drawing->hdc, lineTo->nXStart, lineTo->nYStart, NULL);
@@ -797,34 +797,34 @@ static BOOL gdi_line_to(rdpContext* context, const LINE_TO_ORDER* lineTo)
 
 static BOOL gdi_polyline(rdpContext* context, const POLYLINE_ORDER* polyline)
 {
-	INT32 x = 0;
-	INT32 y = 0;
-	UINT32 color = 0;
-	HGDI_PEN hPen = NULL;
-	DELTA_POINT* points = NULL;
-	rdpGdi* gdi = context->gdi;
-	INT32 w = 0;
-	INT32 h = 0;
+	WINPR_ASSERT(context);
+	WINPR_ASSERT(polyline);
 
+	rdpGdi* gdi = context->gdi;
+	WINPR_ASSERT(gdi);
+
+	UINT32 color = 0;
 	if (!gdi_decode_color(gdi, polyline->penColor, &color, NULL))
 		return FALSE;
 
-	if (!(hPen = gdi_CreatePen(GDI_PS_SOLID, 1, color, gdi->drawing->hdc->format, &gdi->palette)))
+	WINPR_ASSERT(gdi->drawing);
+	WINPR_ASSERT(gdi->drawing->hdc);
+
+	HGDI_PEN hPen = gdi_CreatePen(GDI_PS_SOLID, 1, color, gdi->drawing->hdc->format, &gdi->palette);
+	if (!hPen)
 		return FALSE;
 
 	gdi_SelectObject(gdi->drawing->hdc, (HGDIOBJECT)hPen);
 	gdi_SetROP2(gdi->drawing->hdc, WINPR_ASSERTING_INT_CAST(int32_t, polyline->bRop2));
-	x = polyline->xStart;
-	y = polyline->yStart;
-	gdi_ClipCoords(gdi->drawing->hdc, &x, &y, &w, &h, NULL, NULL);
+	INT32 x = polyline->xStart;
+	INT32 y = polyline->yStart;
 	gdi_MoveToEx(gdi->drawing->hdc, x, y, NULL);
-	points = polyline->points;
+	DELTA_POINT* points = polyline->points;
 
 	for (UINT32 i = 0; i < polyline->numDeltaEntries; i++)
 	{
 		x += points[i].x;
 		y += points[i].y;
-		gdi_ClipCoords(gdi->drawing->hdc, &x, &y, &w, &h, NULL, NULL);
 		gdi_LineTo(gdi->drawing->hdc, x, y);
 		gdi_MoveToEx(gdi->drawing->hdc, x, y, NULL);
 	}
@@ -1068,20 +1068,19 @@ static BOOL gdi_surface_bits(rdpContext* context, const SURFACE_BITS_COMMAND* cm
 	size_t size = 0;
 	REGION16 region;
 	RECTANGLE_16 cmdRect = { 0 };
-	UINT32 nbRects = 0;
-	const RECTANGLE_16* rects = NULL;
 
 	if (!context || !cmd)
 		return FALSE;
 
 	gdi = context->gdi;
-	WLog_Print(
-	    gdi->log, WLOG_DEBUG,
-	    "destLeft %" PRIu32 " destTop %" PRIu32 " destRight %" PRIu32 " destBottom %" PRIu32 " "
-	    "bpp %" PRIu8 " flags %" PRIx8 " codecID %" PRIu16 " width %" PRIu16 " height %" PRIu16
-	    " length %" PRIu32 "",
-	    cmd->destLeft, cmd->destTop, cmd->destRight, cmd->destBottom, cmd->bmp.bpp, cmd->bmp.flags,
-	    cmd->bmp.codecID, cmd->bmp.width, cmd->bmp.height, cmd->bmp.bitmapDataLength);
+	WLog_Print(gdi->log, WLOG_DEBUG,
+	           "destLeft %" PRIu32 " destTop %" PRIu32 " destRight %" PRIu32 " destBottom %" PRIu32
+	           " "
+	           "bpp %" PRIu8 " flags %" PRIx8 " codecID %s [0x%04" PRIu16 "] width %" PRIu16
+	           " height %" PRIu16 " length %" PRIu32 "",
+	           cmd->destLeft, cmd->destTop, cmd->destRight, cmd->destBottom, cmd->bmp.bpp,
+	           cmd->bmp.flags, freerdp_codec_id_to_str(cmd->bmp.codecID), cmd->bmp.codecID,
+	           cmd->bmp.width, cmd->bmp.height, cmd->bmp.bitmapDataLength);
 	region16_init(&region);
 
 	if (!intersect_rect(gdi, cmd, &cmdRect))
@@ -1145,26 +1144,38 @@ static BOOL gdi_surface_bits(rdpContext* context, const SURFACE_BITS_COMMAND* cm
 			break;
 	}
 
-	if (!(rects = region16_rects(&region, &nbRects)))
-		goto out;
-
-	for (UINT32 i = 0; i < nbRects; i++)
 	{
-		UINT32 left = rects[i].left;
-		UINT32 top = rects[i].top;
-		UINT32 width = rects[i].right - rects[i].left;
-		UINT32 height = rects[i].bottom - rects[i].top;
-
-		if (!gdi_InvalidateRegion(gdi->primary->hdc, WINPR_ASSERTING_INT_CAST(int32_t, left),
-		                          WINPR_ASSERTING_INT_CAST(int32_t, top),
-		                          WINPR_ASSERTING_INT_CAST(int32_t, width),
-		                          WINPR_ASSERTING_INT_CAST(int32_t, height)))
-		{
-			WLog_ERR(TAG, "Failed to update invalid region");
+		UINT32 nbRects = 0;
+		const RECTANGLE_16* rects = region16_rects(&region, &nbRects);
+		if (!rects && (nbRects > 0))
 			goto out;
+
+		if (nbRects == 0)
+		{
+			const int32_t w = cmdRect.right - cmdRect.left;
+			const int32_t h = cmdRect.bottom - cmdRect.top;
+			if (!gdi_InvalidateRegion(gdi->primary->hdc, cmdRect.left, cmdRect.top, w, h))
+				goto out;
+		}
+		for (UINT32 i = 0; i < nbRects; i++)
+		{
+			const RECTANGLE_16* rect = &rects[i];
+
+			UINT32 left = rect->left;
+			UINT32 top = rect->top;
+			UINT32 width = rect->right - rect->left;
+			UINT32 height = rect->bottom - rect->top;
+
+			if (!gdi_InvalidateRegion(gdi->primary->hdc, WINPR_ASSERTING_INT_CAST(int32_t, left),
+			                          WINPR_ASSERTING_INT_CAST(int32_t, top),
+			                          WINPR_ASSERTING_INT_CAST(int32_t, width),
+			                          WINPR_ASSERTING_INT_CAST(int32_t, height)))
+			{
+				WLog_ERR(TAG, "Failed to update invalid region");
+				goto out;
+			}
 		}
 	}
-
 	result = TRUE;
 out:
 	region16_uninit(&region);
@@ -1451,25 +1462,29 @@ void gdi_free(freerdp* instance)
 
 BOOL gdi_send_suppress_output(rdpGdi* gdi, BOOL suppress)
 {
-	RECTANGLE_16 rect;
-	rdpSettings* settings = NULL;
-	rdpUpdate* update = NULL;
-
-	if (!gdi || !gdi->context->settings || !gdi->context->update)
+	if (!gdi || !gdi->context)
 		return FALSE;
 
 	if (gdi->suppressOutput == suppress)
 		return TRUE;
 
 	gdi->suppressOutput = suppress;
-	settings = gdi->context->settings;
-	update = gdi->context->update;
-	rect.left = 0;
-	rect.top = 0;
+
+	rdpContext* context = gdi->context;
+	rdpSettings* settings = context->settings;
+	WINPR_ASSERT(settings);
+
+	rdpUpdate* update = context->update;
+	WINPR_ASSERT(update);
 
 	const UINT32 w = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
 	const UINT32 h = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
-	rect.right = WINPR_ASSERTING_INT_CAST(UINT16, w);
-	rect.bottom = WINPR_ASSERTING_INT_CAST(UINT16, h);
-	return update->SuppressOutput(gdi->context, !suppress, &rect);
+
+	const RECTANGLE_16 rect = { .left = 0,
+		                        .top = 0,
+		                        .right = WINPR_ASSERTING_INT_CAST(UINT16, w),
+		                        .bottom = WINPR_ASSERTING_INT_CAST(UINT16, h) };
+
+	WINPR_ASSERT(update->SuppressOutput);
+	return update->SuppressOutput(context, !suppress, &rect);
 }
