@@ -1,9 +1,44 @@
 
 #include <winpr/crt.h>
+#include <winpr/assert.h>
 #include <winpr/sspi.h>
 #include <winpr/print.h>
 #include <winpr/wlog.h>
 
+struct test_input_t
+{
+	const char* user;
+	const char* domain;
+	const char* pwd;
+	const BYTE* ntlm;
+	const BYTE* ntlmv2;
+	BOOL dynamic;
+	BOOL expected;
+};
+
+typedef struct
+{
+	CtxtHandle context;
+	ULONG cbMaxToken;
+	ULONG fContextReq;
+	ULONG pfContextAttr;
+	TimeStamp expiration;
+	PSecBuffer pBuffer;
+	SecBuffer inputBuffer[2];
+	SecBuffer outputBuffer[2];
+	BOOL haveContext;
+	BOOL haveInputBuffer;
+	BOOL UseNtlmV2Hash;
+	LPTSTR ServicePrincipalName;
+	SecBufferDesc inputBufferDesc;
+	SecBufferDesc outputBufferDesc;
+	CredHandle credentials;
+	BOOL confidentiality;
+	SecPkgInfo* pPackageInfo;
+	SecurityFunctionTable* table;
+	SEC_WINNT_AUTH_IDENTITY identity;
+} TEST_NTLM_SERVER;
+
 static BYTE TEST_NTLM_TIMESTAMP[8] = { 0x33, 0x57, 0xbd, 0xb1, 0x07, 0x8b, 0xcf, 0x01 };
 
 static BYTE TEST_NTLM_CLIENT_CHALLENGE[8] = { 0x20, 0xc0, 0x2b, 0x3d, 0xc0, 0x61, 0xa7, 0x73 };
@@ -72,9 +107,16 @@ static const BYTE TEST_NTLM_HASH[16] = { 0xd5, 0x92, 0x2a, 0x65, 0xc4, 0xd5, 0xc
 static const BYTE TEST_NTLM_V2_HASH[16] = { 0x4c, 0x7f, 0x70, 0x6f, 0x7d, 0xde, 0x05, 0xa9,
 	                                        0xd1, 0xa0, 0xf4, 0xe7, 0xff, 0xe3, 0xbf, 0xb8 };
 
+static const BYTE TEST_EMPTY_PWD_NTLM_HASH[] = { 0x31, 0xd6, 0xcf, 0xe0, 0xd1, 0x6a, 0xe9, 0x31,
+	                                             0xb7, 0x3c, 0x59, 0xd7, 0xe0, 0xc0, 0x89, 0xc0 };
+
+static const BYTE TEST_EMPTY_PWD_NTLM_V2_HASH[] = {
+	0x0b, 0xce, 0x54, 0x87, 0x4e, 0x94, 0x20, 0x9e, 0x34, 0x48, 0x97, 0xc1, 0x60, 0x03, 0x6e, 0x3b
+};
+
 #define NTLM_PACKAGE_NAME NTLM_SSP_NAME
 
-struct _TEST_NTLM_CLIENT
+typedef struct
 {
 	CtxtHandle context;
 	ULONG cbMaxToken;
@@ -94,22 +136,29 @@ struct _TEST_NTLM_CLIENT
 	SecPkgInfo* pPackageInfo;
 	SecurityFunctionTable* table;
 	SEC_WINNT_AUTH_IDENTITY identity;
-};
-typedef struct _TEST_NTLM_CLIENT TEST_NTLM_CLIENT;
+} TEST_NTLM_CLIENT;
 
-int test_ntlm_client_init(TEST_NTLM_CLIENT* ntlm, const char* user, const char* domain,
-                          const char* password)
+static int test_ntlm_client_init(TEST_NTLM_CLIENT* ntlm, const char* user, const char* domain,
+                                 const char* password)
 {
-	SECURITY_STATUS status;
+	SECURITY_STATUS status = SEC_E_INTERNAL_ERROR;
+
+	WINPR_ASSERT(ntlm);
+
 	SecInvalidateHandle(&(ntlm->context));
 	ntlm->table = InitSecurityInterfaceEx(TEST_SSPI_INTERFACE);
-	sspi_SetAuthIdentity(&(ntlm->identity), user, domain, password);
+	if (!ntlm->table)
+		return -1;
+	const int rc = sspi_SetAuthIdentity(&(ntlm->identity), user, domain, password);
+	if (rc < 0)
+		return rc;
+
 	status = ntlm->table->QuerySecurityPackageInfo(NTLM_PACKAGE_NAME, &ntlm->pPackageInfo);
 
 	if (status != SEC_E_OK)
 	{
-		fprintf(stderr, "QuerySecurityPackageInfo status: %s (0x%08" PRIX32 ")\n",
-		        GetSecurityStatusString(status), status);
+		(void)fprintf(stderr, "QuerySecurityPackageInfo status: %s (0x%08" PRIX32 ")\n",
+		              GetSecurityStatusString(status), status);
 		return -1;
 	}
 
@@ -120,8 +169,8 @@ int test_ntlm_client_init(TEST_NTLM_CLIENT* ntlm, const char* user, const char*
 
 	if (status != SEC_E_OK)
 	{
-		fprintf(stderr, "AcquireCredentialsHandle status: %s (0x%08" PRIX32 ")\n",
-		        GetSecurityStatusString(status), status);
+		(void)fprintf(stderr, "AcquireCredentialsHandle status: %s (0x%08" PRIX32 ")\n",
+		              GetSecurityStatusString(status), status);
 		return -1;
 	}
 
@@ -130,10 +179,7 @@ int test_ntlm_client_init(TEST_NTLM_CLIENT* ntlm, const char* user, const char*
 	ZeroMemory(&ntlm->inputBuffer, sizeof(SecBuffer));
 	ZeroMemory(&ntlm->outputBuffer, sizeof(SecBuffer));
 	ntlm->fContextReq = 0;
-#if 0
-	/* HTTP authentication flags */
-	ntlm->fContextReq |= ISC_REQ_CONFIDENTIALITY;
-#endif
+
 	/* NLA authentication flags */
 	ntlm->fContextReq |= ISC_REQ_MUTUAL_AUTH;
 	ntlm->fContextReq |= ISC_REQ_CONFIDENTIALITY;
@@ -141,7 +187,7 @@ int test_ntlm_client_init(TEST_NTLM_CLIENT* ntlm, const char* user, const char*
 	return 1;
 }
 
-void test_ntlm_client_uninit(TEST_NTLM_CLIENT* ntlm)
+static void test_ntlm_client_uninit(TEST_NTLM_CLIENT* ntlm)
 {
 	if (!ntlm)
 		return;
@@ -159,9 +205,16 @@ void test_ntlm_client_uninit(TEST_NTLM_CLIENT* ntlm)
 
 	if (ntlm->table)
 	{
-		ntlm->table->FreeCredentialsHandle(&ntlm->credentials);
-		ntlm->table->FreeContextBuffer(ntlm->pPackageInfo);
-		ntlm->table->DeleteSecurityContext(&ntlm->context);
+		SECURITY_STATUS status = ntlm->table->FreeCredentialsHandle(&ntlm->credentials);
+		WINPR_ASSERT((status == SEC_E_OK) || (status == SEC_E_SECPKG_NOT_FOUND) ||
+		             (status == SEC_E_UNSUPPORTED_FUNCTION));
+
+		status = ntlm->table->FreeContextBuffer(ntlm->pPackageInfo);
+		WINPR_ASSERT((status == SEC_E_OK) || (status = SEC_E_INVALID_HANDLE));
+
+		status = ntlm->table->DeleteSecurityContext(&ntlm->context);
+		WINPR_ASSERT((status == SEC_E_OK) || (status == SEC_E_SECPKG_NOT_FOUND) ||
+		             (status == SEC_E_UNSUPPORTED_FUNCTION));
 	}
 }
 
@@ -202,11 +255,37 @@ void test_ntlm_client_uninit(TEST_NTLM_CLIENT* ntlm)
  *                                          (  Client End  )
  *                                           --------------
  */
-
-int test_ntlm_client_authenticate(TEST_NTLM_CLIENT* ntlm)
+static BOOL IsSecurityStatusError(SECURITY_STATUS status)
 {
-	SECURITY_STATUS status;
+	BOOL error = TRUE;
 
+	switch (status)
+	{
+		case SEC_E_OK:
+		case SEC_I_CONTINUE_NEEDED:
+		case SEC_I_COMPLETE_NEEDED:
+		case SEC_I_COMPLETE_AND_CONTINUE:
+		case SEC_I_LOCAL_LOGON:
+		case SEC_I_CONTEXT_EXPIRED:
+		case SEC_I_INCOMPLETE_CREDENTIALS:
+		case SEC_I_RENEGOTIATE:
+		case SEC_I_NO_LSA_CONTEXT:
+		case SEC_I_SIGNATURE_NEEDED:
+		case SEC_I_NO_RENEGOTIATION:
+			error = FALSE;
+			break;
+		default:
+			break;
+	}
+
+	return error;
+}
+
+static int test_ntlm_client_authenticate(TEST_NTLM_CLIENT* ntlm)
+{
+	SECURITY_STATUS status = SEC_E_INTERNAL_ERROR;
+
+	WINPR_ASSERT(ntlm);
 	if (ntlm->outputBuffer[0].pvBuffer)
 	{
 		free(ntlm->outputBuffer[0].pvBuffer);
@@ -233,7 +312,7 @@ int test_ntlm_client_authenticate(TEST_NTLM_CLIENT* ntlm)
 
 	if ((!ntlm) || (!ntlm->table))
 	{
-		fprintf(stderr, "ntlm_authenticate: invalid ntlm context\n");
+		(void)fprintf(stderr, "ntlm_authenticate: invalid ntlm context\n");
 		return -1;
 	}
 
@@ -243,10 +322,18 @@ int test_ntlm_client_authenticate(TEST_NTLM_CLIENT* ntlm)
 	    SECURITY_NATIVE_DREP, (ntlm->haveInputBuffer) ? &ntlm->inputBufferDesc : NULL, 0,
 	    &ntlm->context, &ntlm->outputBufferDesc, &ntlm->pfContextAttr, &ntlm->expiration);
 
+	if (IsSecurityStatusError(status))
+		return -1;
+
 	if ((status == SEC_I_COMPLETE_AND_CONTINUE) || (status == SEC_I_COMPLETE_NEEDED))
 	{
 		if (ntlm->table->CompleteAuthToken)
-			ntlm->table->CompleteAuthToken(&ntlm->context, &ntlm->outputBufferDesc);
+		{
+			SECURITY_STATUS rc =
+			    ntlm->table->CompleteAuthToken(&ntlm->context, &ntlm->outputBufferDesc);
+			if (rc != SEC_E_OK)
+				return -1;
+		}
 
 		if (status == SEC_I_COMPLETE_NEEDED)
 			status = SEC_E_OK;
@@ -264,10 +351,9 @@ int test_ntlm_client_authenticate(TEST_NTLM_CLIENT* ntlm)
 	return (status == SEC_I_CONTINUE_NEEDED) ? 1 : 0;
 }
 
-TEST_NTLM_CLIENT* test_ntlm_client_new()
+static TEST_NTLM_CLIENT* test_ntlm_client_new(void)
 {
-	TEST_NTLM_CLIENT* ntlm;
-	ntlm = (TEST_NTLM_CLIENT*)calloc(1, sizeof(TEST_NTLM_CLIENT));
+	TEST_NTLM_CLIENT* ntlm = (TEST_NTLM_CLIENT*)calloc(1, sizeof(TEST_NTLM_CLIENT));
 
 	if (!ntlm)
 		return NULL;
@@ -275,7 +361,7 @@ TEST_NTLM_CLIENT* test_ntlm_client_new()
 	return ntlm;
 }
 
-void test_ntlm_client_free(TEST_NTLM_CLIENT* ntlm)
+static void test_ntlm_client_free(TEST_NTLM_CLIENT* ntlm)
 {
 	if (!ntlm)
 		return;
@@ -284,42 +370,24 @@ void test_ntlm_client_free(TEST_NTLM_CLIENT* ntlm)
 	free(ntlm);
 }
 
-struct _TEST_NTLM_SERVER
+static int test_ntlm_server_init(TEST_NTLM_SERVER* ntlm)
 {
-	CtxtHandle context;
-	ULONG cbMaxToken;
-	ULONG fContextReq;
-	ULONG pfContextAttr;
-	TimeStamp expiration;
-	PSecBuffer pBuffer;
-	SecBuffer inputBuffer[2];
-	SecBuffer outputBuffer[2];
-	BOOL haveContext;
-	BOOL haveInputBuffer;
-	BOOL UseNtlmV2Hash;
-	LPTSTR ServicePrincipalName;
-	SecBufferDesc inputBufferDesc;
-	SecBufferDesc outputBufferDesc;
-	CredHandle credentials;
-	BOOL confidentiality;
-	SecPkgInfo* pPackageInfo;
-	SecurityFunctionTable* table;
-	SEC_WINNT_AUTH_IDENTITY identity;
-};
-typedef struct _TEST_NTLM_SERVER TEST_NTLM_SERVER;
+	SECURITY_STATUS status = SEC_E_INTERNAL_ERROR;
+
+	WINPR_ASSERT(ntlm);
 
-int test_ntlm_server_init(TEST_NTLM_SERVER* ntlm)
-{
-	SECURITY_STATUS status;
 	ntlm->UseNtlmV2Hash = TRUE;
 	SecInvalidateHandle(&(ntlm->context));
 	ntlm->table = InitSecurityInterfaceEx(TEST_SSPI_INTERFACE);
+	if (!ntlm->table)
+		return SEC_E_INTERNAL_ERROR;
+
 	status = ntlm->table->QuerySecurityPackageInfo(NTLM_PACKAGE_NAME, &ntlm->pPackageInfo);
 
 	if (status != SEC_E_OK)
 	{
-		fprintf(stderr, "QuerySecurityPackageInfo status: %s (0x%08" PRIX32 ")\n",
-		        GetSecurityStatusString(status), status);
+		(void)fprintf(stderr, "QuerySecurityPackageInfo status: %s (0x%08" PRIX32 ")\n",
+		              GetSecurityStatusString(status), status);
 		return -1;
 	}
 
@@ -330,8 +398,8 @@ int test_ntlm_server_init(TEST_NTLM_SERVER* ntlm)
 
 	if (status != SEC_E_OK)
 	{
-		fprintf(stderr, "AcquireCredentialsHandle status: %s (0x%08" PRIX32 ")\n",
-		        GetSecurityStatusString(status), status);
+		(void)fprintf(stderr, "AcquireCredentialsHandle status: %s (0x%08" PRIX32 ")\n",
+		              GetSecurityStatusString(status), status);
 		return -1;
 	}
 
@@ -351,7 +419,7 @@ int test_ntlm_server_init(TEST_NTLM_SERVER* ntlm)
 	return 1;
 }
 
-void test_ntlm_server_uninit(TEST_NTLM_SERVER* ntlm)
+static void test_ntlm_server_uninit(TEST_NTLM_SERVER* ntlm)
 {
 	if (!ntlm)
 		return;
@@ -369,15 +437,22 @@ void test_ntlm_server_uninit(TEST_NTLM_SERVER* ntlm)
 
 	if (ntlm->table)
 	{
-		ntlm->table->FreeCredentialsHandle(&ntlm->credentials);
-		ntlm->table->FreeContextBuffer(ntlm->pPackageInfo);
-		ntlm->table->DeleteSecurityContext(&ntlm->context);
+		SECURITY_STATUS status = ntlm->table->FreeCredentialsHandle(&ntlm->credentials);
+		WINPR_ASSERT(status == SEC_E_OK);
+		status = ntlm->table->FreeContextBuffer(ntlm->pPackageInfo);
+		WINPR_ASSERT(status == SEC_E_OK);
+		status = ntlm->table->DeleteSecurityContext(&ntlm->context);
+		WINPR_ASSERT(status == SEC_E_OK);
 	}
 }
 
-int test_ntlm_server_authenticate(TEST_NTLM_SERVER* ntlm)
+static int test_ntlm_server_authenticate(const struct test_input_t* targ, TEST_NTLM_SERVER* ntlm)
 {
-	SECURITY_STATUS status;
+	SECURITY_STATUS status = SEC_E_INTERNAL_ERROR;
+
+	WINPR_ASSERT(ntlm);
+	WINPR_ASSERT(targ);
+
 	ntlm->inputBufferDesc.ulVersion = SECBUFFER_VERSION;
 	ntlm->inputBufferDesc.cBuffers = 1;
 	ntlm->inputBufferDesc.pBuffers = ntlm->inputBuffer;
@@ -397,49 +472,30 @@ int test_ntlm_server_authenticate(TEST_NTLM_SERVER* ntlm)
 	    ntlm->fContextReq, SECURITY_NATIVE_DREP, &ntlm->context, &ntlm->outputBufferDesc,
 	    &ntlm->pfContextAttr, &ntlm->expiration);
 
-	if ((status == SEC_I_COMPLETE_AND_CONTINUE) || (status == SEC_I_COMPLETE_NEEDED))
+	if (status == SEC_I_CONTINUE_NEEDED)
 	{
-		SecPkgContext_AuthIdentity AuthIdentity;
-		SecPkgContext_AuthNtlmHash AuthNtlmHash;
-		ZeroMemory(&AuthIdentity, sizeof(SecPkgContext_AuthIdentity));
-		ZeroMemory(&AuthNtlmHash, sizeof(SecPkgContext_AuthNtlmHash));
-		status = ntlm->table->QueryContextAttributes(&ntlm->context, SECPKG_ATTR_AUTH_IDENTITY,
-		                                             &AuthIdentity);
+		SecPkgContext_AuthNtlmHash AuthNtlmHash = { 0 };
 
-		if (status == SEC_E_OK)
+		if (ntlm->UseNtlmV2Hash)
 		{
-			if (strcmp(AuthIdentity.User, TEST_NTLM_USER) == 0)
-			{
-				if (ntlm->UseNtlmV2Hash)
-				{
-					AuthNtlmHash.Version = 2;
-					CopyMemory(AuthNtlmHash.NtlmHash, TEST_NTLM_V2_HASH, 16);
-				}
-				else
-				{
-					AuthNtlmHash.Version = 1;
-					CopyMemory(AuthNtlmHash.NtlmHash, TEST_NTLM_HASH, 16);
-				}
-
-				status = ntlm->table->SetContextAttributes(
-				    &ntlm->context, SECPKG_ATTR_AUTH_NTLM_HASH, &AuthNtlmHash,
-				    sizeof(SecPkgContext_AuthNtlmHash));
-			}
+			AuthNtlmHash.Version = 2;
+			CopyMemory(AuthNtlmHash.NtlmHash, targ->ntlmv2, 16);
+		}
+		else
+		{
+			AuthNtlmHash.Version = 1;
+			CopyMemory(AuthNtlmHash.NtlmHash, targ->ntlm, 16);
 		}
 
-		if (ntlm->table->CompleteAuthToken)
-			status = ntlm->table->CompleteAuthToken(&ntlm->context, &ntlm->outputBufferDesc);
-
-		if (status == SEC_I_COMPLETE_NEEDED)
-			status = SEC_E_OK;
-		else if (status == SEC_I_COMPLETE_AND_CONTINUE)
-			status = SEC_I_CONTINUE_NEEDED;
+		status =
+		    ntlm->table->SetContextAttributes(&ntlm->context, SECPKG_ATTR_AUTH_NTLM_HASH,
+		                                      &AuthNtlmHash, sizeof(SecPkgContext_AuthNtlmHash));
 	}
 
 	if ((status != SEC_E_OK) && (status != SEC_I_CONTINUE_NEEDED))
 	{
-		fprintf(stderr, "AcceptSecurityContext status: %s (0x%08" PRIX32 ")\n",
-		        GetSecurityStatusString(status), status);
+		(void)fprintf(stderr, "AcceptSecurityContext status: %s (0x%08" PRIX32 ")\n",
+		              GetSecurityStatusString(status), status);
 		return -1; /* Access Denied */
 	}
 
@@ -447,10 +503,9 @@ int test_ntlm_server_authenticate(TEST_NTLM_SERVER* ntlm)
 	return (status == SEC_I_CONTINUE_NEEDED) ? 1 : 0;
 }
 
-TEST_NTLM_SERVER* test_ntlm_server_new()
+static TEST_NTLM_SERVER* test_ntlm_server_new(void)
 {
-	TEST_NTLM_SERVER* ntlm;
-	ntlm = (TEST_NTLM_SERVER*)calloc(1, sizeof(TEST_NTLM_SERVER));
+	TEST_NTLM_SERVER* ntlm = (TEST_NTLM_SERVER*)calloc(1, sizeof(TEST_NTLM_SERVER));
 
 	if (!ntlm)
 		return NULL;
@@ -458,7 +513,7 @@ TEST_NTLM_SERVER* test_ntlm_server_new()
 	return ntlm;
 }
 
-void test_ntlm_server_free(TEST_NTLM_SERVER* ntlm)
+static void test_ntlm_server_free(TEST_NTLM_SERVER* ntlm)
 {
 	if (!ntlm)
 		return;
@@ -467,49 +522,46 @@ void test_ntlm_server_free(TEST_NTLM_SERVER* ntlm)
 	free(ntlm);
 }
 
-int TestNTLM(int argc, char* argv[])
+static BOOL test_default(const struct test_input_t* arg)
 {
-	int status;
-	PSecBuffer pSecBuffer;
-	TEST_NTLM_CLIENT* client;
-	TEST_NTLM_SERVER* server;
-	BOOL DynamicTest = TRUE;
+	BOOL rc = FALSE;
+	PSecBuffer pSecBuffer = NULL;
+
+	WINPR_ASSERT(arg);
+
+	printf("testcase {user=%s, domain=%s, password=%s, dynamic=%s}\n", arg->user, arg->domain,
+	       arg->pwd, arg->dynamic ? "TRUE" : "FALSE");
+
 	/**
 	 * Client Initialization
 	 */
-	client = test_ntlm_client_new();
+	TEST_NTLM_CLIENT* client = test_ntlm_client_new();
+	TEST_NTLM_SERVER* server = test_ntlm_server_new();
 
-	if (!client)
+	if (!client || !server)
 	{
-		printf("Memory allocation failed");
-		return -1;
+		printf("Memory allocation failed\n");
+		goto fail;
 	}
 
-	status = test_ntlm_client_init(client, TEST_NTLM_USER, TEST_NTLM_DOMAIN, TEST_NTLM_PASSWORD);
+	int status = test_ntlm_client_init(client, arg->user, arg->domain, arg->pwd);
 
 	if (status < 0)
 	{
 		printf("test_ntlm_client_init failure\n");
-		return -1;
+		goto fail;
 	}
 
 	/**
 	 * Server Initialization
 	 */
-	server = test_ntlm_server_new();
-
-	if (!server)
-	{
-		printf("Memory allocation failed\n");
-		return -1;
-	}
 
 	status = test_ntlm_server_init(server);
 
 	if (status < 0)
 	{
 		printf("test_ntlm_server_init failure\n");
-		return -1;
+		goto fail;
 	}
 
 	/**
@@ -520,50 +572,58 @@ int TestNTLM(int argc, char* argv[])
 	if (status < 0)
 	{
 		printf("test_ntlm_client_authenticate failure\n");
-		return -1;
+		goto fail;
 	}
 
-	if (!DynamicTest)
+	if (!arg->dynamic)
 	{
-		SecPkgContext_AuthNtlmTimestamp AuthNtlmTimestamp;
-		SecPkgContext_AuthNtlmClientChallenge AuthNtlmClientChallenge;
-		SecPkgContext_AuthNtlmServerChallenge AuthNtlmServerChallenge;
+		SecPkgContext_AuthNtlmTimestamp AuthNtlmTimestamp = { 0 };
+		SecPkgContext_AuthNtlmClientChallenge AuthNtlmClientChallenge = { 0 };
+		SecPkgContext_AuthNtlmServerChallenge AuthNtlmServerChallenge = { 0 };
 		CopyMemory(AuthNtlmTimestamp.Timestamp, TEST_NTLM_TIMESTAMP, 8);
 		AuthNtlmTimestamp.ChallengeOrResponse = TRUE;
-		client->table->SetContextAttributes(&client->context, SECPKG_ATTR_AUTH_NTLM_TIMESTAMP,
-		                                    &AuthNtlmTimestamp,
-		                                    sizeof(SecPkgContext_AuthNtlmTimestamp));
+		SECURITY_STATUS rc = client->table->SetContextAttributes(
+		    &client->context, SECPKG_ATTR_AUTH_NTLM_TIMESTAMP, &AuthNtlmTimestamp,
+		    sizeof(SecPkgContext_AuthNtlmTimestamp));
+		WINPR_ASSERT((rc == SEC_E_OK) || (rc == SEC_E_SECPKG_NOT_FOUND));
+
 		AuthNtlmTimestamp.ChallengeOrResponse = FALSE;
-		client->table->SetContextAttributes(&client->context, SECPKG_ATTR_AUTH_NTLM_TIMESTAMP,
-		                                    &AuthNtlmTimestamp,
-		                                    sizeof(SecPkgContext_AuthNtlmTimestamp));
+		rc = client->table->SetContextAttributes(&client->context, SECPKG_ATTR_AUTH_NTLM_TIMESTAMP,
+		                                         &AuthNtlmTimestamp,
+		                                         sizeof(SecPkgContext_AuthNtlmTimestamp));
+		WINPR_ASSERT((rc == SEC_E_OK) || (rc == SEC_E_SECPKG_NOT_FOUND));
+
 		CopyMemory(AuthNtlmClientChallenge.ClientChallenge, TEST_NTLM_CLIENT_CHALLENGE, 8);
 		CopyMemory(AuthNtlmServerChallenge.ServerChallenge, TEST_NTLM_SERVER_CHALLENGE, 8);
-		client->table->SetContextAttributes(
+		rc = client->table->SetContextAttributes(
 		    &client->context, SECPKG_ATTR_AUTH_NTLM_CLIENT_CHALLENGE, &AuthNtlmClientChallenge,
 		    sizeof(SecPkgContext_AuthNtlmClientChallenge));
-		client->table->SetContextAttributes(
+		WINPR_ASSERT((rc == SEC_E_OK) || (rc == SEC_E_SECPKG_NOT_FOUND));
+
+		rc = client->table->SetContextAttributes(
 		    &client->context, SECPKG_ATTR_AUTH_NTLM_SERVER_CHALLENGE, &AuthNtlmServerChallenge,
 		    sizeof(SecPkgContext_AuthNtlmServerChallenge));
+		WINPR_ASSERT((rc == SEC_E_OK) || (rc == SEC_E_SECPKG_NOT_FOUND));
 	}
 
 	pSecBuffer = &(client->outputBuffer[0]);
 
-	if (!DynamicTest)
+	if (!arg->dynamic)
 	{
 		pSecBuffer->cbBuffer = sizeof(TEST_NTLM_NEGOTIATE) - 1;
-		pSecBuffer->pvBuffer = (void*)malloc(pSecBuffer->cbBuffer);
+		free(pSecBuffer->pvBuffer);
+		pSecBuffer->pvBuffer = malloc(pSecBuffer->cbBuffer);
 
 		if (!pSecBuffer->pvBuffer)
 		{
 			printf("Memory allocation failed\n");
-			return -1;
+			goto fail;
 		}
 
 		CopyMemory(pSecBuffer->pvBuffer, TEST_NTLM_NEGOTIATE, pSecBuffer->cbBuffer);
 	}
 
-	fprintf(stderr, "NTLM_NEGOTIATE (length = %" PRIu32 "):\n", pSecBuffer->cbBuffer);
+	(void)fprintf(stderr, "NTLM_NEGOTIATE (length = %" PRIu32 "):\n", pSecBuffer->cbBuffer);
 	winpr_HexDump("sspi.test", WLOG_DEBUG, (BYTE*)pSecBuffer->pvBuffer, pSecBuffer->cbBuffer);
 	/**
 	 * Server <- Negotiate Message
@@ -573,62 +633,72 @@ int TestNTLM(int argc, char* argv[])
 	server->inputBuffer[0].BufferType = SECBUFFER_TOKEN;
 	server->inputBuffer[0].pvBuffer = pSecBuffer->pvBuffer;
 	server->inputBuffer[0].cbBuffer = pSecBuffer->cbBuffer;
-	status = test_ntlm_server_authenticate(server);
+	status = test_ntlm_server_authenticate(arg, server);
 
 	if (status < 0)
 	{
 		printf("test_ntlm_server_authenticate failure\n");
-		return -1;
+		goto fail;
 	}
 
-	if (!DynamicTest)
+	if (!arg->dynamic)
 	{
-		SecPkgContext_AuthNtlmTimestamp AuthNtlmTimestamp;
-		SecPkgContext_AuthNtlmClientChallenge AuthNtlmClientChallenge;
-		SecPkgContext_AuthNtlmServerChallenge AuthNtlmServerChallenge;
+		SecPkgContext_AuthNtlmTimestamp AuthNtlmTimestamp = { 0 };
+		SecPkgContext_AuthNtlmClientChallenge AuthNtlmClientChallenge = { 0 };
+		SecPkgContext_AuthNtlmServerChallenge AuthNtlmServerChallenge = { 0 };
 		CopyMemory(AuthNtlmTimestamp.Timestamp, TEST_NTLM_TIMESTAMP, 8);
 		AuthNtlmTimestamp.ChallengeOrResponse = TRUE;
-		client->table->SetContextAttributes(&server->context, SECPKG_ATTR_AUTH_NTLM_TIMESTAMP,
-		                                    &AuthNtlmTimestamp,
-		                                    sizeof(SecPkgContext_AuthNtlmTimestamp));
+		SECURITY_STATUS rc = client->table->SetContextAttributes(
+		    &server->context, SECPKG_ATTR_AUTH_NTLM_TIMESTAMP, &AuthNtlmTimestamp,
+		    sizeof(SecPkgContext_AuthNtlmTimestamp));
+		WINPR_ASSERT(rc == SEC_E_OK);
+
 		AuthNtlmTimestamp.ChallengeOrResponse = FALSE;
-		client->table->SetContextAttributes(&server->context, SECPKG_ATTR_AUTH_NTLM_TIMESTAMP,
-		                                    &AuthNtlmTimestamp,
-		                                    sizeof(SecPkgContext_AuthNtlmTimestamp));
+		rc = client->table->SetContextAttributes(&server->context, SECPKG_ATTR_AUTH_NTLM_TIMESTAMP,
+		                                         &AuthNtlmTimestamp,
+		                                         sizeof(SecPkgContext_AuthNtlmTimestamp));
+		WINPR_ASSERT(rc == SEC_E_OK);
+
 		CopyMemory(AuthNtlmClientChallenge.ClientChallenge, TEST_NTLM_CLIENT_CHALLENGE, 8);
 		CopyMemory(AuthNtlmServerChallenge.ServerChallenge, TEST_NTLM_SERVER_CHALLENGE, 8);
-		server->table->SetContextAttributes(
+		rc = server->table->SetContextAttributes(
 		    &server->context, SECPKG_ATTR_AUTH_NTLM_CLIENT_CHALLENGE, &AuthNtlmClientChallenge,
 		    sizeof(SecPkgContext_AuthNtlmClientChallenge));
-		server->table->SetContextAttributes(
+		WINPR_ASSERT(rc == SEC_E_OK);
+
+		rc = server->table->SetContextAttributes(
 		    &server->context, SECPKG_ATTR_AUTH_NTLM_SERVER_CHALLENGE, &AuthNtlmServerChallenge,
 		    sizeof(SecPkgContext_AuthNtlmServerChallenge));
+		WINPR_ASSERT(rc == SEC_E_OK);
 	}
 
 	pSecBuffer = &(server->outputBuffer[0]);
 
-	if (!DynamicTest)
+	if (!arg->dynamic)
 	{
-		SecPkgContext_AuthNtlmMessage AuthNtlmMessage;
+		SecPkgContext_AuthNtlmMessage AuthNtlmMessage = { 0 };
 		pSecBuffer->cbBuffer = sizeof(TEST_NTLM_CHALLENGE) - 1;
-		pSecBuffer->pvBuffer = (void*)malloc(pSecBuffer->cbBuffer);
+		free(pSecBuffer->pvBuffer);
+		pSecBuffer->pvBuffer = malloc(pSecBuffer->cbBuffer);
 
 		if (!pSecBuffer->pvBuffer)
 		{
 			printf("Memory allocation failed\n");
-			return -1;
+			goto fail;
 		}
 
 		CopyMemory(pSecBuffer->pvBuffer, TEST_NTLM_CHALLENGE, pSecBuffer->cbBuffer);
 		AuthNtlmMessage.type = 2;
 		AuthNtlmMessage.length = pSecBuffer->cbBuffer;
 		AuthNtlmMessage.buffer = (BYTE*)pSecBuffer->pvBuffer;
-		server->table->SetContextAttributes(&server->context, SECPKG_ATTR_AUTH_NTLM_MESSAGE,
-		                                    &AuthNtlmMessage,
-		                                    sizeof(SecPkgContext_AuthNtlmMessage));
+		SECURITY_STATUS rc = server->table->SetContextAttributes(
+		    &server->context, SECPKG_ATTR_AUTH_NTLM_MESSAGE, &AuthNtlmMessage,
+		    sizeof(SecPkgContext_AuthNtlmMessage));
+		if (rc != SEC_E_OK)
+			goto fail;
 	}
 
-	fprintf(stderr, "NTLM_CHALLENGE (length = %" PRIu32 "):\n", pSecBuffer->cbBuffer);
+	(void)fprintf(stderr, "NTLM_CHALLENGE (length = %" PRIu32 "):\n", pSecBuffer->cbBuffer);
 	winpr_HexDump("sspi.test", WLOG_DEBUG, (BYTE*)pSecBuffer->pvBuffer, pSecBuffer->cbBuffer);
 	/**
 	 * Client <- Challenge Message
@@ -643,26 +713,27 @@ int TestNTLM(int argc, char* argv[])
 	if (status < 0)
 	{
 		printf("test_ntlm_client_authenticate failure\n");
-		return -1;
+		goto fail;
 	}
 
 	pSecBuffer = &(client->outputBuffer[0]);
 
-	if (!DynamicTest)
+	if (!arg->dynamic)
 	{
 		pSecBuffer->cbBuffer = sizeof(TEST_NTLM_AUTHENTICATE) - 1;
-		pSecBuffer->pvBuffer = (void*)malloc(pSecBuffer->cbBuffer);
+		free(pSecBuffer->pvBuffer);
+		pSecBuffer->pvBuffer = malloc(pSecBuffer->cbBuffer);
 
 		if (!pSecBuffer->pvBuffer)
 		{
 			printf("Memory allocation failed\n");
-			return -1;
+			goto fail;
 		}
 
 		CopyMemory(pSecBuffer->pvBuffer, TEST_NTLM_AUTHENTICATE, pSecBuffer->cbBuffer);
 	}
 
-	fprintf(stderr, "NTLM_AUTHENTICATE (length = %" PRIu32 "):\n", pSecBuffer->cbBuffer);
+	(void)fprintf(stderr, "NTLM_AUTHENTICATE (length = %" PRIu32 "):\n", pSecBuffer->cbBuffer);
 	winpr_HexDump("sspi.test", WLOG_DEBUG, (BYTE*)pSecBuffer->pvBuffer, pSecBuffer->cbBuffer);
 	/**
 	 * Server <- Authenticate Message
@@ -671,18 +742,51 @@ int TestNTLM(int argc, char* argv[])
 	server->inputBuffer[0].BufferType = SECBUFFER_TOKEN;
 	server->inputBuffer[0].pvBuffer = pSecBuffer->pvBuffer;
 	server->inputBuffer[0].cbBuffer = pSecBuffer->cbBuffer;
-	status = test_ntlm_server_authenticate(server);
+	status = test_ntlm_server_authenticate(arg, server);
 
 	if (status < 0)
 	{
 		printf("test_ntlm_server_authenticate failure\n");
-		return -1;
+		goto fail;
 	}
 
+	rc = TRUE;
+
+fail:
 	/**
 	 * Cleanup & Termination
 	 */
 	test_ntlm_client_free(client);
 	test_ntlm_server_free(server);
-	return 0;
+
+	printf("testcase {user=%s, domain=%s, password=%s, dynamic=%s} returns %d\n", arg->user,
+	       arg->domain, arg->pwd, arg->dynamic ? "TRUE" : "FALSE", rc);
+	return rc;
+}
+
+int TestNTLM(int argc, char* argv[])
+{
+	WINPR_UNUSED(argc);
+	WINPR_UNUSED(argv);
+
+	const struct test_input_t inputs[] = {
+		{ TEST_NTLM_USER, TEST_NTLM_DOMAIN, TEST_NTLM_PASSWORD, TEST_NTLM_HASH, TEST_NTLM_V2_HASH,
+		  TRUE, TRUE },
+		{ TEST_NTLM_USER, TEST_NTLM_DOMAIN, TEST_NTLM_PASSWORD, TEST_NTLM_HASH, TEST_NTLM_V2_HASH,
+		  FALSE, TRUE },
+		{ TEST_NTLM_USER, TEST_NTLM_DOMAIN, "", TEST_EMPTY_PWD_NTLM_HASH,
+		  TEST_EMPTY_PWD_NTLM_V2_HASH, TRUE, TRUE },
+		{ TEST_NTLM_USER, TEST_NTLM_DOMAIN, NULL, TEST_EMPTY_PWD_NTLM_HASH,
+		  TEST_EMPTY_PWD_NTLM_V2_HASH, TRUE, FALSE }
+	};
+
+	int rc = 0;
+	for (size_t x = 0; x < ARRAYSIZE(inputs); x++)
+	{
+		const struct test_input_t* cur = &inputs[x];
+		const BOOL res = test_default(cur);
+		if (res != cur->expected)
+			rc = -1;
+	}
+	return rc;
 }
